# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, import  (GUICG24)
import GemRB
import CharGenCommon

#import from a game 
ImportWindow = 0

def OnLoad():
	global ImportWindow

	ImportWindow = GemRB.LoadWindow(20, "GUICG")
	CharGenCommon.PositionCharGenWin(ImportWindow)

	TextAreaControl = ImportWindow.GetControl(4)
	TextAreaControl.SetText(53774)

	TextAreaControl = ImportWindow.GetControl(2)
#Fill TextArea Control with character sheets, make textarea a listbox

	DoneButton = ImportWindow.GetControl(0)
	DoneButton.SetText(2610)
	DoneButton.SetDisabled(True)
	DoneButton.MakeDefault()

	CancelButton = ImportWindow.GetControl(1)
	CancelButton.SetText(15416)
	CancelButton.MakeEscape()

	DoneButton.OnPress (Done1Press)
	CancelButton.OnPress (CancelPress)
	ImportWindow.Focus()
	return

def Done1Press():
	DoneButton = ImportWindow.GetControl(0)
	DoneButton.SetText(11973)
	DoneButton.SetState(IE_GUI_BUTTON_DISABLED)
	DoneButton.OnPress (Done2Press)
	return
	
def Done2Press():
	if ImportWindow:
		ImportWindow.Close ()
	GemRB.SetNextScript("Start")
	return

def CancelPress():
	if ImportWindow:
		ImportWindow.Close ()
	GemRB.SetNextScript("CharGen")
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUIREC.py - scripts to control character record windows from GUIREC winpack

###################################################

import GemRB
import GUICommon
import CommonTables
import GUICommonWindows
import GUIRECCommon
import IDLUCommon
import LUCommon
from GUIDefines import *
from GUICommonWindows import CreateTopWinLoader, ToggleWindow, OpenWindowOnce
from ie_stats import *
from ie_restype import *
from ie_feats import FEAT_ARMORED_ARCANA

Topic = None
HelpTable = None
DescTable = None
RecordsWindow = None
RecordsTextArea = None
InformationWindow = None

BonusSpellTable = None
HateRaceTable = None

if not BonusSpellTable:
	BonusSpellTable = GemRB.LoadTable ("mxsplbon")
if not HateRaceTable:
	HateRaceTable = GemRB.LoadTable ("haterace")

# class level stats
Classes = IDLUCommon.Levels

#don't allow exporting polymorphed or dead characters
def Exportable(pc):
	if not (GemRB.GetPlayerStat (pc, IE_MC_FLAGS)&MC_EXPORTABLE): return False
	if GemRB.GetPlayerStat (pc, IE_POLYMORPHED): return False
	if GemRB.GetPlayerStat (pc, IE_STATE_ID)&STATE_DEAD: return False
	return True

def InitRecordsWindow (Window):
	global RecordsWindow
	global BonusSpellTable, HateRaceTable

	RecordsWindow = Window

	#portrait icon
	Button = Window.GetControl (2)
	Button.SetState (IE_GUI_BUTTON_LOCKED)
	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_PICTURE, OP_SET)

	#information (help files)
	Button = Window.GetControl (1)
	Button.SetText (11946)
	Button.OnPress (OpenHelpWindow)

	#biography
	Button = Window.GetControl (59)
	Button.SetText (18003)
	Button.OnPress (GUIRECCommon.OpenBiographyWindow)

	#export
	Button = Window.GetControl (36)
	Button.SetText (13956)
	Button.OnPress (GUIRECCommon.OpenExportWindow)

	#customize
	Button = Window.GetControl (50)
	Button.SetText (10645)
	Button.OnPress (GUIRECCommon.OpenCustomizeWindow)

	#general
	Button = Window.GetControl (60)
	Button.SetTooltip (40316)
	Button.SetVarAssoc ("SelectWindow", 1)
	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
	Button.OnPress (lambda: UpdateRecordsWindow(Window))

	#weapons and armour
	Button = Window.GetControl (61)
	Button.SetTooltip (40317)
	Button.SetVarAssoc ("SelectWindow", 2)
	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
	Button.OnPress (lambda: UpdateRecordsWindow(Window))

	#skills and feats
	Button = Window.GetControl (62)
	Button.SetTooltip (40318)
	Button.SetVarAssoc ("SelectWindow", 3)
	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
	Button.OnPress (lambda: UpdateRecordsWindow(Window))

	#miscellaneous
	Button = Window.GetControl (63)
	Button.SetTooltip (33500)
	Button.SetVarAssoc ("SelectWindow", 4)
	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
	Button.OnPress (lambda: UpdateRecordsWindow(Window))

	GemRB.SetVar ("SelectWindow", 1)

	#level up
	Button = Window.GetControl (37)
	Button.SetText (7175)
	Button.OnPress (OpenLevelUpWindow)

	return

def ColorDiff (Window, Label, diff):
	Color = {'r' : 0, 'g' : 0, 'b' : 0}
	if diff>0:
		Color['g'] = 255
	elif diff<0:
		Color['r'] = 255
	else:
		Color = {'r' : 255, 'g' : 255, 'b' : 255}

	Label.SetColor (Color)
	return

def ColorDiff2 (Window, Label, diff):
	if diff:
		Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
	else:
		Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
	return

def GetBonusSpells (pc):
	bonusSpells = {}
	classes = []
	# cheack each class/kit
	for i in range(11):
		level = GemRB.GetPlayerStat (pc, Classes[i])
		if not level:
			continue

		ClassTitle = GUICommonWindows.GetActorClassTitle (pc, i)
		# find the casting stat
		ClassName = GUICommon.GetClassRowName (i, "index")
		Stat = CommonTables.ClassSkills.GetValue (ClassName, "CASTING")
		if Stat == "*":
			continue
		Stat = GemRB.GetPlayerStat (pc, Stat)
		if Stat < 12: # boni start with positive modifiers
			continue

		# get max spell level we can cast, since only usable boni are displayed
		# check the relevant mxspl* table
		SpellTable = CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL")
		if SpellTable == "*":
			SpellTable = CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL")
		SpellTable = GemRB.LoadTable (SpellTable)
		maxLevel = 0
		for i in range(SpellTable.GetColumnCount()):
			spells = SpellTable.GetValue (str(level), str(i+1)) # not all tables start at 1, so use a named lookup
			if not spells:
				break
			maxLevel = i+1

		classes.append(ClassTitle)
		# check if at casting stat size, there is any bonus spell in BonusSpellTable
		bonusSpells[ClassTitle] = [0] * maxLevel
		for level in range (1, maxLevel+1):
			bonusSpells[ClassTitle][level-1] = BonusSpellTable.GetValue (Stat-12, level-1)

	return bonusSpells, classes

def HasClassFeatures (pc):
	#clerics turning
	if GemRB.GetPlayerStat(pc, IE_TURNUNDEADLEVEL):
		return True
	#thieves backstabbing damage
	if GemRB.GetPlayerStat(pc, IE_BACKSTABDAMAGEMULTIPLIER):
		return True
	#paladins healing
	if GemRB.GetPlayerStat(pc, IE_LAYONHANDSAMOUNT):
		return True
	# wisdom ac bonus, wholeness of body
	if GemRB.GetPlayerStat (pc, IE_LEVELMONK):
		return True
	return False

def DisplayFavouredEnemy (pc, RangerLevel, second, RecordsTextArea):
	RaceID = 0
	if second == -1:
		RaceID = GemRB.GetPlayerStat(pc, IE_HATEDRACE)
	else:
		RaceID = GemRB.GetPlayerStat(pc, IE_HATEDRACE2+second)
	if RaceID:
		FavouredIndex = HateRaceTable.FindValue (1, RaceID)
		if FavouredIndex is None:
			return
		FavouredName = HateRaceTable.GetValue (FavouredIndex, 0)
		if second == -1:
			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel + 4) // 5), 0))
		else:
			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel + 4) // 5 - second - 1), 0))

def GetFavoredClass (pc, code):
	if GemRB.GetPlayerStat (pc, IE_SEX)==1:
		code = code&15
	else:	
		code = (code>>8)&15

	return code-1

# returns the effective character level modifier
def GetECL (pc):
	RaceIndex = IDLUCommon.GetRace (pc)
	if RaceIndex == None:
		return 0
	RaceRowName = CommonTables.Races.GetRowName (RaceIndex)
	return CommonTables.Races.GetValue (RaceRowName, "ECL")

#class is ignored
def GetNextLevelExp (Level, Adjustment, string=0):
	if Adjustment>5:
		Adjustment = 5
	if (Level < CommonTables.NextLevel.GetColumnCount (4) - 5):
		exp = CommonTables.NextLevel.GetValue (4, Level + Adjustment)
		if string:
			return str(exp)
		return exp

	if string:
		return GemRB.GetString(24342) #godhood
	return 0

def DisplayCommon (pc):
	Window = RecordsWindow

	Label = Window.GetControl (0x1000000f)
	Label.SetText (GUICommonWindows.GetActorRaceTitle (pc))

	Button = Window.GetControl (36)
	if Exportable (pc):
		Button.SetState (IE_GUI_BUTTON_ENABLED)
	else:
		Button.SetState (IE_GUI_BUTTON_DISABLED)
	return

def DisplaySavingThrows (pc, RecordsTextArea):
	RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(17379) + "[/color]\n")

	tmp = GemRB.GetPlayerStat (pc, IE_SAVEFORTITUDE)
	RecordsTextArea.Append (DelimitedText(17380, PlusMinusStat(tmp), 0))

	tmp = GemRB.GetPlayerStat (pc, IE_SAVEREFLEX)
	RecordsTextArea.Append (DelimitedText(17381, PlusMinusStat(tmp), 0))

	tmp = GemRB.GetPlayerStat (pc, IE_SAVEWILL)
	RecordsTextArea.Append (DelimitedText(17382, PlusMinusStat(tmp), 0))

# screenshots at http:// lparchive.org/Icewind-Dale-2/Update%2013/
def GNZS(pc, s1, st, force=False):
	value = GemRB.GetPlayerStat (pc, st)
	if value or force:
		RecordsTextArea.Append (DelimitedText(s1, value, 0))
	return

def DisplayGeneral (pc, targetTextArea):
	global RecordsTextArea
	RecordsTextArea = targetTextArea

	#levels
	# get special level penalty for subrace
	adj = GetECL (pc)
	levelsum = GemRB.GetPlayerStat (pc, IE_CLASSLEVELSUM)
	RecordsTextArea.Append ("[color=ffff00]" + GemRB.GetString(40308) + " - " +
                            GemRB.GetString(40309) + ": " + str(levelsum) + "[/color]\n")

	#the class name for highest
	highest = None
	tmp = 0
	for i in range(11):
		level = GemRB.GetPlayerStat (pc, Classes[i])

		if level:
			Class = GUICommonWindows.GetActorClassTitle (pc, i )
			RecordsTextArea.Append (DelimitedText (Class, level, 0))
			if tmp<level:
				highest = i
				tmp = level

	RecordsTextArea.Append ("\n")
	#effective character level
	if adj:
		RecordsTextArea.Append (DelimitedText (40311, levelsum+adj, 0))

	#favoured class
	#get the subrace value
	RaceIndex = IDLUCommon.GetRace (pc)
	Race = CommonTables.Races.GetValue (RaceIndex, 2)
	tmp = CommonTables.Races.GetValue (RaceIndex, 8)

	if tmp == -1:
		tmp = highest
	else:
		tmp = GetFavoredClass(pc, tmp)

	tmp = CommonTables.Classes.GetValue (CommonTables.Classes.GetRowName(tmp), "NAME_REF")
	RecordsTextArea.Append (DelimitedStrRefs (40310, tmp, 0))

	#experience
	RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(17089) + "[/color]\n")

	xp = GemRB.GetPlayerStat (pc, IE_XP)
	RecordsTextArea.Append (DelimitedText (36928, xp, 0, ""))
	tmp = GetNextLevelExp (levelsum, adj, 1)
	RecordsTextArea.Append (DelimitedText (17091, tmp, 0))
	# Multiclassing penalty
	tmp = GemRB.GetMultiClassPenalty (pc)
	if tmp != 0:
		GemRB.SetToken ("XPPENALTY", str(tmp)+"%")
		RecordsTextArea.Append (DelimitedText (39418, "", 0, ""))

	#current effects
	effects = GemRB.GetPlayerStates (pc)
	if len(effects):
		RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(32052) + "[/color]\n")
		StateTable = GemRB.LoadTable ("statdesc")
		for c in effects:
			tmp = StateTable.GetValue (str(c - 66), "DESCRIPTION", GTV_REF)
			RecordsTextArea.Append ("[cap][int=" + str(c) + "]%[/cap][p]" + tmp + "[/p]")

	# TODO: Active Feats (eg. Power attack 4)

	#race
	RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(1048) + "[/color]\n")
	RecordsTextArea.Append ("[p]" + GemRB.GetString(Race) + "[/p]")

	#alignment
	RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(1049) + "[/color]\n")
	tmp = CommonTables.Aligns.FindValue (3, GemRB.GetPlayerStat (pc, IE_ALIGNMENT))
	Align = CommonTables.Aligns.GetValue (tmp, 2, GTV_REF)
	RecordsTextArea.Append ("[p]" + Align + "[/p]")

	#saving throws
	DisplaySavingThrows (pc, RecordsTextArea)

	#class features
	if HasClassFeatures(pc):
		RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(40314) + "[/color]\n")
		tmp = GemRB.GetPlayerStat (pc, IE_TURNUNDEADLEVEL)
		if tmp:
			RecordsTextArea.Append (DelimitedText (12126, tmp, 0))
		# 1d6 at level 1 and +1d6 every two extra rogue levels
		tmp = GemRB.GetPlayerStat (pc, IE_LEVELTHIEF)
		if tmp:
			tmp = (tmp+1)//2
			RecordsTextArea.Append (DelimitedText (24898, str(tmp)+"d6", 0))
		tmp = GemRB.GetPlayerStat (pc, IE_LAYONHANDSAMOUNT)
		if tmp:
			RecordsTextArea.Append (DelimitedText (12127, tmp, 0))
		MonkLevel = GemRB.GetPlayerStat (pc, IE_LEVELMONK)
		if MonkLevel:
			AC = GemRB.GetCombatDetails(pc, 0)["AC"]
			GemRB.SetToken ("number", PlusMinusStat (AC["Wisdom"]))
			RecordsTextArea.Append (DelimitedText (39431, "", 0, ""))
			# wholeness of body
			RecordsTextArea.Append (DelimitedText (39749, MonkLevel*2, 0))

	# favoured enemies; eg Goblins: +2 & Harpies: +1
	RangerLevel = GemRB.GetPlayerStat (pc, IE_LEVELRANGER)
	if RangerLevel:
		RangerString = "\n[color=ffff00]"
		if RangerLevel > 5:
			RangerString += GemRB.GetString (15982)
		else:
			RangerString += GemRB.GetString (15897)
		RecordsTextArea.Append (RangerString + "[/color]\n")
		DisplayFavouredEnemy (pc, RangerLevel, -1, RecordsTextArea)
		for i in range (7):
			DisplayFavouredEnemy (pc, RangerLevel, i, RecordsTextArea)

	#bonus spells
	bonusSpells, classes = GetBonusSpells(pc)
	if len(bonusSpells):
		RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(10344) + "[/color]\n")
		for c in classes:
			if not len(bonusSpells[c]):
				continue
			# class/kit name
			RecordsTextArea.Append ("[p]" + GemRB.GetString (c) + "[/p]")
			for level in range(len(bonusSpells[c])):
				AddIndent()
				# Level X: +Y
				RecordsTextArea.Append (DelimitedText(7192, "+" + str(bonusSpells[c][level]), 0, " " + str(level+1)+": "))

	#ability statistics
	RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(40315) + "[/color]\n")

	# Weight Allowance
	tmp = GemRB.GetAbilityBonus( IE_STR, 3, GemRB.GetPlayerStat(pc, IE_STR) )
	RecordsTextArea.Append (DelimitedText (10338, str(tmp) + " lb."))
	# constitution bonus to hitpoints
	tmp = GUICommon.GetAbilityBonus(pc, IE_CON)
	RecordsTextArea.Append (DelimitedText (10342, PlusMinusStat(tmp)))

	# Magic
	GNZS(pc, 15581, IE_RESISTMAGIC, True)
	#Fire
	GNZS(pc, 14012, IE_RESISTFIRE)
	#Magic Fire
	GNZS(pc, 14077, IE_RESISTMAGICFIRE)
	#Cold
	GNZS(pc, 14014, IE_RESISTCOLD)
	#Magic Cold
	GNZS(pc, 14078, IE_RESISTMAGICCOLD)
	#Electricity
	GNZS(pc, 14013, IE_RESISTELECTRICITY)
	#Acid
	GNZS(pc, 14015, IE_RESISTACID)
	#Spell
	GNZS(pc, 40319, IE_MAGICDAMAGERESISTANCE)
	# Missile
	GNZS(pc, 11767, IE_RESISTMISSILE)
	# Slashing
	GNZS(pc, 11768, IE_RESISTSLASHING)
	# Piercing
	GNZS(pc, 11769, IE_RESISTPIERCING)
	# Crushing
	GNZS(pc, 11770, IE_RESISTCRUSHING)
	# Poison
	GNZS(pc, 14017, IE_RESISTPOISON)

	# Damage Reduction - Spell Effect: Damage Reduction [436] (plain boni are mentioned above as resistances)
	BaseString = 0
	DisplayedHeader = 0
	for damageType in 0, 1:
		if damageType:
			# missile
			BaseString = 11767
		else:
			# general (physical)
			BaseString = 40316

		# cheat a bit and use the reduction at +10 as a modifier to remove from all other values
		# it will hold all the non-enchantment related resistance (there are no +10 weapons)
		mod = GemRB.GetDamageReduction (pc, damageType, 10)
		if mod == -1:
			mod = 0
		for enchantment in range(6):
			damage = GemRB.GetDamageReduction (pc, damageType, enchantment)
			if damage == -1 or damage-mod <= 0:
				continue
			if not DisplayedHeader:
				RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(39325) + "[/color]\n")
				DisplayedHeader = 1

			enchantment += 1 # since we were checking what is allowable, not what bypasses it
			if enchantment:
				enchantment = "+" + str(enchantment)
			else:
				enchantment = "-"
			reduction = "%d/%s" %(damage-mod, enchantment)
			RecordsTextArea.Append (DelimitedText(BaseString, reduction))

	return

# some of the displayed stats are manually indented
def AddIndent():
	RecordsTextArea.Append ("   ")

def PlusMinusStat(value):
	if value >= 0:
		return "+" + str(value)
	return str(value)

def CascadeToHit(total, ac, apr, slot):
	cascade = PlusMinusStat(total)
	babDec = 5
	if ac["Wisdom"] and slot == 10: # fist slot - nothing is equipped
		babDec = 3

	for i in range(1, apr):
		if total-i*babDec > 0: # skips negative ones, meaning a lower number of attacks can be displayed
			cascade = cascade  + "/" + PlusMinusStat(total-i*babDec)
	return cascade

def ToHitOfHand(combatdet, dualwielding, left=0):
	ac = combatdet["AC"]
	tohit = combatdet["ToHitStats"]

	# display all the (successive) attack values (+15/+10/+5)
	if left:
		apr = 1 # offhand gives just one extra attack
		hits = CascadeToHit(tohit["Total"], ac, apr, combatdet["Slot"])
		RecordsTextArea.Append (DelimitedText (733, hits, 0))
	else:
		# account for the fact that the total apr contains the one for the other hand too
		if dualwielding:
			apr = combatdet["APR"]//2 - 1
		else:
			apr = combatdet["APR"]//2
		hits = CascadeToHit(tohit["Total"], ac, apr, combatdet["Slot"])
		RecordsTextArea.Append (DelimitedText (734, hits, 0))

	# Base
	AddIndent()
	hits = CascadeToHit(tohit["Base"], ac, apr, combatdet["Slot"])
	RecordsTextArea.Append (DelimitedText (31353, hits, 0))
	# Weapon bonus
	if tohit["Weapon"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (32560, PlusMinusStat(tohit["Weapon"]), 0))
	# Proficiency bonus
	if tohit["Proficiency"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (32561, PlusMinusStat(tohit["Proficiency"]), 0))
	# Armor Penalty
	if tohit["Armor"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (39816, PlusMinusStat(tohit["Armor"]), 0))
	# Shield Penalty (if you don't have the shield proficiency feat)
	if tohit["Shield"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (39822, PlusMinusStat(tohit["Shield"]), 0))
	# Ability  bonus
	if tohit["Ability"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (33547, PlusMinusStat(tohit["Ability"]), 0))
	# Others
	if tohit["Generic"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (33548, PlusMinusStat(tohit["Generic"]), 0))
	RecordsTextArea.Append ("\n")

def WeaponOfHand(pc, combatdet, dualwielding, left=0):
	slot = combatdet["Slot"]
	slot_item = GemRB.GetSlotItem (pc, slot, 1)
	if not slot_item:
		print("ARGHH, no slot item at slot %d, bailing out!" %(combatdet["Slot"]))
		return
	item = GemRB.GetItem (slot_item["ItemResRef"])

	ammo = None
	if not left: # only the main hand can have ranged weapons, even one-handed
		ammoslot = GemRB.GetEquippedAmmunition (pc)
		if ammoslot != -1:
			ammosi = GemRB.GetSlotItem (pc, ammoslot)
			ammo = GemRB.GetItem (ammosi["ItemResRef"])

	# Main Hand - weapon name
	#  or Ranged - ammo
	name = "ItemName"
	if slot_item['Flags'] & IE_INV_ITEM_IDENTIFIED:
		name = "ItemNameIdentified"
	if combatdet["Flags"]&15 == 2 and ammo: # this is basically wi.wflags & WEAPON_STYLEMASK == WEAPON_RANGED
		RecordsTextArea.Append (DelimitedStrRefs (41123, ammo[name], 0, " - "))
	else:
		if dualwielding and left:
			RecordsTextArea.Append (DelimitedStrRefs (733, item[name], 0, " - "))
		else:
			RecordsTextArea.Append (DelimitedStrRefs (734, item[name], 0, " - "))

	# don't spoil it for unidentified items
	# Unidentified - Details Unknown
	if not (slot_item['Flags'] & IE_INV_ITEM_IDENTIFIED):
		AddIndent()
		RecordsTextArea.Append (DelimitedText(41121, "", 0, ""))
		return

	# Damage
	# display the unresolved damage string (2d6)
	# this is ammo, launcher details come later
	wdice = combatdet["HitHeaderNumDice"]
	wsides = combatdet["HitHeaderDiceSides"]
	wbonus = combatdet["HitHeaderDiceBonus"]
	AddIndent()
	if wbonus:
		RecordsTextArea.Append (DelimitedText (39518, str (wdice)+"d"+str(wsides)+PlusMinusStat(wbonus), 0))
	else:
		RecordsTextArea.Append (DelimitedText (39518, str (wdice)+"d"+str(wsides), 0))
	# any extended headers with damage, eg. Fire: +1d6, which is also computed for the total (00arow08)
	alldos = combatdet["DamageOpcodes"]
	dosmin = 0
	dosmax = 0
	for dos in alldos:
		ddice = dos["NumDice"]
		dsides = dos["DiceSides"]
		dbonus = dos["DiceBonus"]
		dchance = dos["Chance"]
		AddIndent()
		# only display the chance when it isn't 100%
		if dchance == 100:
			dchance = ""
			dosmin += ddice + dbonus
		else:
			dchance = " (%d%%)" % dchance
		dicestr = ""
		if ddice:
			dicestr = "+%dd%d" %(ddice, dsides)

		dicestr = dos["TypeName"].title() + ": " + dicestr
		if dbonus:
			dicestr += PlusMinusStat(dbonus) + dchance
		else:
			dicestr += dchance
		RecordsTextArea.Append ("[p]" + dicestr + "[/p]")

		dosmax += ddice*dsides + dbonus

	# Strength
	abonus = combatdet["WeaponStrBonus"]
	if abonus:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (1145, PlusMinusStat (abonus), 0))
	# Proficiency (bonus)
	pbonus = combatdet["ProfDmgBon"]
	if pbonus:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (32561, PlusMinusStat(pbonus), 0))
	# Launcher
	lbonus = combatdet["LauncherDmgBon"]
	if lbonus:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (41408, PlusMinusStat(lbonus), 0))
	#TODO: Power Attack has its own row
	# Damage Potential (2-12)
	# add any other bonus to the ammo damage calc
	AddIndent()
	wmin = wdice + wbonus + abonus + pbonus + lbonus + dosmin
	wmax = wdice*wsides + wbonus + abonus + pbonus + lbonus + dosmax
	RecordsTextArea.Append (DelimitedText (41120, str (wmin)+"-"+str(wmax), 0))
	# Critical Hit (19-20 / x2)
	crange = combatdet["CriticalRange"]
	cmulti = combatdet["CriticalMultiplier"]
	if crange == 20:
		crange = "20 / x" + str(cmulti)
	else:
		crange = str(crange) + "-20 / x" + str(cmulti)
	AddIndent()
	RecordsTextArea.Append (DelimitedText (41122, crange, 0))
	if not left and dualwielding:
		RecordsTextArea.Append ("\n")

def DisplayWeapons (pc):
	GS = lambda s, pc=pc: GemRB.GetPlayerStat (pc, s)

	###################
	# Attack Roll Modifiers
	RecordsTextArea.Append ("[color=ffff00]" + GemRB.GetString(9457) + "[/color]\n")

	combatdet = GemRB.GetCombatDetails(pc, 0)
	combatdet2 = combatdet # placeholder for the potential offhand numbers
	ac = combatdet["AC"]
	dualwielding = GemRB.IsDualWielding(pc)
	if dualwielding:
		combatdet2 = GemRB.GetCombatDetails(pc, 1)

	# Main Hand
	ToHitOfHand (combatdet, dualwielding)

	# Off Hand
	if dualwielding:
		ToHitOfHand (combatdet2, dualwielding, 1)


	###################
	# Number of Attacks
	if dualwielding:
		# only one extra offhand attack and it is displayed as eg. 2+1
		RecordsTextArea.Append (DelimitedText (9458, str (combatdet["APR"]//2-1)+"+1"))
	else:
		RecordsTextArea.Append (DelimitedText (9458, str (combatdet["APR"]//2)))

	###################
	# Armor Class
	RecordsTextArea.Append ("[color=ffff00]" + GemRB.GetString(33553) + "[/color]\n")
	RecordsTextArea.Append (DelimitedText (33553, str (GS(IE_ARMORCLASS)), 0)) # same as ac["Total"]

	# Base
	AddIndent()
	RecordsTextArea.Append (DelimitedText (31353, str (ac["Natural"]), 0))
	# Armor
	if ac["Armor"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (11997, PlusMinusStat (ac["Armor"]), 0))
	# Shield
	if ac["Shield"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (6347, PlusMinusStat (ac["Shield"]), 0))
	# Deflection
	if ac["Deflection"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (33551, PlusMinusStat (ac["Deflection"]), 0))
	# Generic
	if ac["Generic"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (33552, PlusMinusStat (ac["Generic"]), 0))
	# Dexterity
	if ac["Dexterity"]:
		AddIndent()
		RecordsTextArea.Append (DelimitedText (1151, PlusMinusStat (ac["Dexterity"]), 0))
	# Monk Wisdom Bonus: <number> to AC
	if ac["Wisdom"]:
		GemRB.SetToken ("number", PlusMinusStat (ac["Wisdom"]))
		AddIndent()
		RecordsTextArea.Append (DelimitedText (39431, "", 0, ""))

	###################
	# Armor Class Modifiers
	stat = GS (IE_ACMISSILEMOD) + GS (IE_ACSLASHINGMOD) + GS (IE_ACPIERCINGMOD) + GS (IE_ACCRUSHINGMOD)
	if stat:
		RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(11766) + "[/color]\n")

		# Missile
		if GS (IE_ACMISSILEMOD):
			AddIndent()
			RecordsTextArea.Append (DelimitedText (11767, PlusMinusStat(GS (IE_ACMISSILEMOD)), 0))
		# Slashing
		if GS (IE_ACSLASHINGMOD):
			AddIndent()
			RecordsTextArea.Append (DelimitedText (11768, PlusMinusStat (GS (IE_ACSLASHINGMOD)), 0))
		# Piercing
		if GS (IE_ACPIERCINGMOD):
			AddIndent()
			RecordsTextArea.Append (DelimitedText (11769, PlusMinusStat (GS (IE_ACPIERCINGMOD)), 0))
		# Bludgeoning
		if GS (IE_ACCRUSHINGMOD):
			AddIndent()
			RecordsTextArea.Append (DelimitedText (11770, PlusMinusStat (GS (IE_ACCRUSHINGMOD))))

	###################
	# Arcane spell failure
	if GS(IE_LEVELBARD) + GS(IE_LEVELSORCERER) + GS(IE_LEVELMAGE):
		RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(41391) + "[/color]\n")

		# Casting Failure
		failure = GemRB.GetSpellFailure (pc)
		total = failure["Total"]
		arcana = 5*GemRB.HasFeat(pc, FEAT_ARMORED_ARCANA)
		armor = 5 * (failure["Armor"] + failure["Shield"])
		other = armor - arcana
		# account for armored arcana only if it doesn't cause a negative value (eating into the misc bonus)
		if other < 0:
			other = total
		else:
			other = total - other

		if total < 0:
			total = 0
		RecordsTextArea.Append (DelimitedText (41390, str (total)+"%", 0))
		# Armor Penalty (same as for skills and everything else)
		if failure["Armor"]:
			AddIndent()
			RecordsTextArea.Append (DelimitedText (39816, PlusMinusStat(5*failure["Armor"])+"%", 0))
		# Shield Penalty
		if failure["Shield"]:
			AddIndent()
			RecordsTextArea.Append (DelimitedText (39822, PlusMinusStat(5*failure["Shield"])+"%", 0))
		if armor and arcana:
			AddIndent()
			RecordsTextArea.Append (DelimitedText (36352, PlusMinusStat(-arcana)+"%", 0))
		# Other, just a guess to show the remainder
		if other:
			AddIndent()
			RecordsTextArea.Append (DelimitedText (33548, PlusMinusStat(other)+"%", 0))

	###################
	# Weapon Statistics
	RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(41119) + "[/color]\n")

	# Main hand
	WeaponOfHand(pc, combatdet, dualwielding)

	# Off-hand (if any)
	if dualwielding:
		WeaponOfHand(pc, combatdet2, dualwielding, 1)

	DisplaySavingThrows (pc, RecordsTextArea)

	return

def DisplaySkills (pc, SkillsArea):
	
	def PrintStatTable (title, tabname):
		feats = True if tabname == "feats" else False
		lookup = "featreq" if feats else "skillsta"

		nameTab = GemRB.LoadTable (tabname)
		itemTab = GemRB.LoadTable (lookup)
		rows = itemTab.GetRowCount ()
		
		SkillsArea.Append ("[color=ffff00]" + title + "[/color]\n")
		
		items = []
		for i in range(rows):
			item = itemTab.GetValue (i, 0, GTV_STAT)
			name = nameTab.GetValue (i, 1, GTV_REF)

			if feats and GemRB.HasFeat(pc, i):
				value = (name,) if not item else (name, GemRB.GetPlayerStat (pc, item),)
				items.append (value)
			elif not feats:
				value = GemRB.GetPlayerStat (pc, item)
				base = GemRB.GetPlayerStat (pc, item, 1)
				untrained = nameTab.GetValue (i, 3)
				# only show positive (modified) skills that either don't require training or had it already
				if (untrained and value == base and value > 0):
					items.append((name, str(value),))
				elif (value>0 and untrained) or (not untrained and base):
					items.append((name, str(value) + " (" + str(base) + ")",))

		items.sort()
		for item in items:
			if len(item) > 1:
				SkillsArea.Append ("[p]" + item[0] + ": " + str(item[1]) + "[/p]")
			else:
				SkillsArea.Append ("[p]" + item[0] + "[/p]")
		return

	PrintStatTable (GemRB.GetString(11983), "skills")
	SkillsArea.Append ("\n")
	PrintStatTable (GemRB.GetString(36361), "feats")

	return

def DelimitedStrRefs(strref1, strref2, newlines=1, delimiter=": "):
	text = GemRB.GetString(strref2) 
	return DelimitedText(strref1, text, newlines, delimiter)

def DelimitedText(strref, text, newlines=1, delimiter=": "):
	return "[p]" + GemRB.GetString(strref) + delimiter + str(text) + "[/p]" + ("\n" * newlines)

#character information
def DisplayMisc (pc):
	TotalPartyExp = 0
	TotalCount = 0
	for i in range (1, GemRB.GetPartySize() + 1):
		stat = GemRB.GetPCStats(i)
		TotalPartyExp = TotalPartyExp + stat['KillsTotalXP']
		TotalCount = TotalCount + stat['KillsTotalCount']

	stat = GemRB.GetPCStats (pc)

	#favourites
	RecordsTextArea.Append ("[color=ffff00]" + GemRB.GetString(40320) + "[/color]\n")

	#favourite spell and weapon
	RecordsTextArea.Append (DelimitedStrRefs (11949, stat['FavouriteSpell'], 0))
	RecordsTextArea.Append (DelimitedStrRefs (11950, stat['FavouriteWeapon'], 0))

	# combat details
	RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(40322) + "[/color]\n")

	#most powerful vanquished, time spent, xp and kills
	RecordsTextArea.Append (DelimitedStrRefs (11947, stat['BestKilledName'], 0))

	time = GUICommon.SetCurrentDateTokens (stat)
	RecordsTextArea.Append (DelimitedText (11948, time, 0))

	# Experience Value of Kills
	RecordsTextArea.Append (DelimitedText (11953, stat['KillsTotalXP'], 0))

	# Number of Kills
	RecordsTextArea.Append (DelimitedText (11954, stat['KillsTotalCount'], 0))

	# Total Experience Value in Party
	if TotalPartyExp:
		val = stat['KillsTotalXP'] * 100 // TotalPartyExp
	else:
		val = 0
	RecordsTextArea.Append (DelimitedText (11951, str(val) + "%", 0))

	# Percentage of Total Kills in Party
	if TotalPartyExp:
		val = stat['KillsTotalCount'] * 100 // TotalCount
	else:
		val = 0
	RecordsTextArea.Append (DelimitedText (11952, str(val) + "%", 0))

	return

def UpdateRecordsWindow (Window):
	global RecordsTextArea

	pc = GemRB.GameGetSelectedPCSingle ()

	#name
	Label = Window.GetControl (0x1000000e)
	Label.SetText (GemRB.GetPlayerName (pc, 0))

	#portrait
	Button = Window.GetControl (2)
	Button.SetPicture (GemRB.GetPlayerPortrait (pc, 0)["Sprite"])

	# armorclass
	Label = Window.GetControl (0x10000028)
	Label.SetText (str (GemRB.GetPlayerStat (pc, IE_ARMORCLASS)))
	Label.SetTooltip (17183)

	# hp now
	Label = Window.GetControl (0x10000029)
	Label.SetText (str (GemRB.GetPlayerStat (pc, IE_HITPOINTS)))
	Label.SetTooltip (17184)

	# hp max
	Label = Window.GetControl (0x1000002a)
	Label.SetText (str (GemRB.GetPlayerStat (pc, IE_MAXHITPOINTS)))
	Label.SetTooltip (17378)

	#level up
	Button = Window.GetControl (37)
	levelsum = GemRB.GetPlayerStat (pc, IE_CLASSLEVELSUM)
	if GetNextLevelExp(levelsum, GetECL(pc)) <= GemRB.GetPlayerStat (pc, IE_XP):
		Button.SetState (IE_GUI_BUTTON_ENABLED)
	else:
		Button.SetState (IE_GUI_BUTTON_DISABLED)

	# stats

	sstr = GemRB.GetPlayerStat (pc, IE_STR)
	dstr = sstr-GemRB.GetPlayerStat (pc, IE_STR,1)
	bstr = GUICommon.GetAbilityBonus (pc, IE_STR)
	sint = GemRB.GetPlayerStat (pc, IE_INT)
	dint = sint-GemRB.GetPlayerStat (pc, IE_INT,1)
	bint = GUICommon.GetAbilityBonus (pc, IE_INT)
	swis = GemRB.GetPlayerStat (pc, IE_WIS)
	dwis = swis-GemRB.GetPlayerStat (pc, IE_WIS,1)
	bwis = GUICommon.GetAbilityBonus (pc, IE_WIS)
	sdex = GemRB.GetPlayerStat (pc, IE_DEX)
	ddex = sdex-GemRB.GetPlayerStat (pc, IE_DEX,1)
	bdex = GUICommon.GetAbilityBonus (pc, IE_DEX)
	scon = GemRB.GetPlayerStat (pc, IE_CON)
	dcon = scon-GemRB.GetPlayerStat (pc, IE_CON,1)
	bcon = GUICommon.GetAbilityBonus (pc, IE_CON)
	schr = GemRB.GetPlayerStat (pc, IE_CHR)
	dchr = schr-GemRB.GetPlayerStat (pc, IE_CHR,1)
	bchr = GUICommon.GetAbilityBonus (pc, IE_CHR)

	Label = Window.GetControl (0x1000002f)
	Label.SetText (str(sstr))
	ColorDiff2 (Window, Label, dstr)

	Label = Window.GetControl (0x10000009)
	Label.SetText (str(sdex))
	ColorDiff2 (Window, Label, ddex)

	Label = Window.GetControl (0x1000000a)
	Label.SetText (str(scon))
	ColorDiff2 (Window, Label, dcon)

	Label = Window.GetControl (0x1000000b)
	Label.SetText (str(sint))
	ColorDiff2 (Window, Label, dint)

	Label = Window.GetControl (0x1000000c)
	Label.SetText (str(swis))
	ColorDiff2 (Window, Label, dwis)

	Label = Window.GetControl (0x1000000d)
	Label.SetText (str(schr))
	ColorDiff2 (Window, Label, dchr)

	Label = Window.GetControl (0x10000035)
	Label.SetText (PlusMinusStat(bstr))
	ColorDiff (Window, Label, bstr)

	Label = Window.GetControl (0x10000036)
	Label.SetText (PlusMinusStat(bdex))
	ColorDiff (Window, Label, bdex)

	Label = Window.GetControl (0x10000037)
	Label.SetText (PlusMinusStat(bcon))
	ColorDiff (Window, Label, bcon)

	Label = Window.GetControl (0x10000038)
	Label.SetText (PlusMinusStat(bint))
	ColorDiff (Window, Label, bint)

	Label = Window.GetControl (0x10000039)
	Label.SetText (PlusMinusStat(bwis))
	ColorDiff (Window, Label, bwis)

	Label = Window.GetControl (0x1000003a)
	Label.SetText (PlusMinusStat(bchr))
	ColorDiff (Window, Label, bchr)

	RecordsTextArea = Window.GetControl (45)
	RecordsTextArea.SetText ("")
	RecordsTextArea.SetColor (ColorWhitish, TA_COLOR_INITIALS)

	DisplayCommon (pc)

	SelectWindow = GemRB.GetVar ("SelectWindow")
	if SelectWindow == 1:
		DisplayGeneral (pc, RecordsTextArea)
	elif SelectWindow == 2:
		DisplayWeapons (pc)
	elif SelectWindow == 3:
		DisplaySkills (pc, RecordsTextArea)
	elif SelectWindow == 4:
		DisplayMisc (pc)

	#if actor is uncontrollable, make this grayed
	GUICommon.AdjustWindowVisibility (Window, pc, 0)

	return

ToggleRecordsWindow = CreateTopWinLoader(2, "GUIREC", ToggleWindow, InitRecordsWindow, UpdateRecordsWindow, True)
OpenRecordsWindow = CreateTopWinLoader(2, "GUIREC", OpenWindowOnce, InitRecordsWindow, UpdateRecordsWindow, True)

def CloseHelpWindow ():
	global DescTable, InformationWindow

	if InformationWindow:
		InformationWindow.Close ()
		InformationWindow = None
	if DescTable:
		DescTable = None
	return

#ingame help
def OpenHelpWindow ():
	global HelpTable, InformationWindow

	InformationWindow = Window = GemRB.LoadWindow (57, "GUIREC")

	HelpTable = GemRB.LoadTable ("topics")
	GemRB.SetVar("Topic", 0)
	GemRB.SetVar("TopIndex", 0)

	for i in range(11):
		title = HelpTable.GetValue (i, 0)
		Button = Window.GetControl (i+27)
		Label = Window.GetControl (i+0x10000004)

		Button.SetVarAssoc ("Topic", i)
		if title:
			Label.SetText (title)
			Button.SetState (IE_GUI_BUTTON_LOCKED)
			Button.OnPress (UpdateHelpWindow)
		else:
			Label.SetText ("")
			Button.SetState (IE_GUI_BUTTON_DISABLED)

	#done
	Button = Window.GetControl (1)
	Button.SetText (11973)
	Button.OnPress (CloseHelpWindow)
	Window.ShowModal (MODAL_SHADOW_GRAY)
	UpdateHelpWindow ()
	return

def UpdateHelpWindow ():
	global DescTable, Topic

	Window = InformationWindow

	if Topic!=GemRB.GetVar ("Topic"):
		Topic = GemRB.GetVar ("Topic")
		GemRB.SetVar ("TopIndex",0)
		GemRB.SetVar ("Selected",0)

	for i in range(11):
		#Button = Window.GetControl (i+27)
		Label = Window.GetControl (i+0x10000004)
		if Topic==i:
			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
		else:
			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})

	resource = HelpTable.GetValue (Topic, 1)
	if DescTable:
		DescTable = None

	if resource:
		DescTable = GemRB.LoadTable (resource)

	startrow = HelpTable.GetValue (Topic, 4)
	if startrow<0:
		i = -startrow - 10
	else:
		if DescTable:
			i = DescTable.GetRowCount () - 10 - startrow
		if i < 0: i = 0

	ScrollBar = Window.GetControl (4)
	ScrollBar.SetVarAssoc ("TopIndex", i)
	ScrollBar.OnChange (RefreshHelpWindow)

	RefreshHelpWindow ()
	return

def RefreshHelpWindow ():
	Window = InformationWindow
	Topic = GemRB.GetVar ("Topic")
	TopIndex = GemRB.GetVar ("TopIndex")
	Selected = GemRB.GetVar ("Selected")

	titlecol = HelpTable.GetValue (Topic, 2)
	desccol = HelpTable.GetValue (Topic, 3)
	startrow = HelpTable.GetValue (Topic, 4)
	if startrow<0: startrow = 0

	for i in range(11):
		Button = Window.GetControl (i+71)
		Label = Window.GetControl (i+0x10000030)

		if i+TopIndex==Selected:
			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
		else:
			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
		title = ""
		if DescTable:
			title = DescTable.GetValue (i+startrow+TopIndex, titlecol)
		if title != "*":
			Label.SetText (title)
			Button.SetState (IE_GUI_BUTTON_LOCKED)
			Button.SetVarAssoc ("Selected", i+TopIndex)
			Button.OnPress (RefreshHelpWindow)
		else:
			Label.SetText ("")
			Button.SetState (IE_GUI_BUTTON_DISABLED)

	if Selected<0 or DescTable == None:
		desc=""
	else:
		desc = DescTable.GetValue (Selected+startrow, desccol)

	TextArea = Window.GetControl (2)
	TextArea.SetText (desc)
	return

LUWindow = None
LUKitWindow = None
LevelDiff = 1
def CloseLUWindow ():
	global LUWindow, LUKitWindow

	if LUKitWindow:
		LUKitWindow.Close ()
		LUKitWindow = None

	if LUWindow:
		LUWindow.Close ()
		LUWindow = None

def OpenLevelUpWindow ():
	global LUWindow, LevelDiff

	LUWindow = Window = GemRB.LoadWindow (54, "GUIREC")

	# Figure out the level difference
	# the original ignored all but the fighter row in xplevel.2da
	# we do the same, since IE_CLASS becomes useless for mc actors
	# (iwd2 never updates it; it's not a bitfield like IE_KIT)
	pc = GemRB.GameGetSelectedPCSingle ()
	xp = GemRB.GetPlayerStat (pc, IE_XP)
	nextLevel = LUCommon.GetNextLevelFromExp (xp, 5)
	levelSum = GemRB.GetPlayerStat (pc, IE_CLASSLEVELSUM)
	LevelDiff = nextLevel - levelSum - GetECL(pc)

	# next
	Button = Window.GetControl (0)
	Button.SetText (36789)
	Button.OnPress (OpenLUKitWindow)
	Button.SetState (IE_GUI_BUTTON_DISABLED)
	Button.MakeDefault ()

	# static Class selection
	#Label = Window.GetControl (0x10000000)
	#Label.SetText (7175)

	# 2-12 are the class name buttons
	# 15-25 are the class level labels
	GemRB.SetVar ("LUClass", None)
	for i in range(2,13):
		Button = Window.GetControl (i)
		Label = Window.GetControl (0x10000000 + i+13)

		ClassTitle = CommonTables.Classes.GetRowName (i-2)
		ClassTitle = CommonTables.Classes.GetValue (ClassTitle, "NAME_REF", GTV_REF)
		Button.SetText (ClassTitle)
		level = GemRB.GetPlayerStat (pc, Classes[i-2])
		if level > 0:
			Label.SetText (str(level))

		# disable monks/paladins due to order restrictions?
		specflag = GemRB.GetPlayerStat (pc, IE_SPECFLAGS)
		if specflag&SPECF_MONKOFF and i == 7:
			Button.SetState (IE_GUI_BUTTON_DISABLED)
		elif specflag&SPECF_PALADINOFF and i == 8:
			Button.SetState (IE_GUI_BUTTON_DISABLED)
		else:
			Button.SetState (IE_GUI_BUTTON_ENABLED)
		Button.SetVarAssoc ("LUClass", i-2)
		Button.OnPress (LUClassPress)
		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)

	# description
	TextArea = Window.GetControl (13)
	TextArea.SetText (17242)

	# 14 scrollbar
	# 14 static Level label
	# 26 does not exist

	# cancel
	Button = Window.GetControl (27)
	Button.SetText (13727)
	Button.OnPress (CloseLUWindow)

	Window.ShowModal (MODAL_SHADOW_NONE)

OldLevelLabel = 0
def LUClassPress ():
	global OldLevelLabel

	Window = LUWindow
	Button = Window.GetControl (0)
	Button.SetState (IE_GUI_BUTTON_ENABLED)

	# display class info
	TextArea = Window.GetControl (13)
	i = GemRB.GetVar ("LUClass") or 0
	ClassDesc = CommonTables.Classes.GetRowName (i)
	ClassDesc = CommonTables.Classes.GetValue (ClassDesc, "DESC_REF", GTV_REF)
	TextArea.SetText (ClassDesc)

	# increase/decrease level label by LevelDiff (usually 1)
	if OldLevelLabel != i:
		pc = GemRB.GameGetSelectedPCSingle ()
		j = OldLevelLabel
		Label = Window.GetControl (0x10000000 + j+15)
		level = Label.QueryText ()
		if level == "":
			level = 1
		level = int(level)
		if level-LevelDiff > 0:
			Label.SetText (str(level - LevelDiff))
		else:
			Label.SetText ("")

		OldLevelLabel = i
		Label = Window.GetControl (0x10000000 + i+15)
		level = GemRB.GetPlayerStat (pc, Classes[i])
		Label.SetText (str(level + LevelDiff))

# if the pc is a paladin or a monk, multiclassing is limited;
# only their school's favoured class is fair game, otherwise
# they lose access to further paladin/monk progression
def HandleSpecFlagExclusion(pc, lucls, lukit):
	# RESEARCH: it's unclear what happened if you multiclassed into a (monk) from a bad class
	monkLevel = GemRB.GetPlayerStat (pc, IE_LEVELMONK)
	paladinLevel = GemRB.GetPlayerStat (pc, IE_LEVELPALADIN)
	if monkLevel == 0 and paladinLevel == 0 and lucls != 6 and lucls != 7:
		return

	# TODO: unhardcode this kit->kit/class dict
	# paladins: ilmater, helm->fighter, mystra->wizard
	# monks: old->rogue, broken->ilmater, dark moon->sorcerer
	orderFavs = { 1: 0x8000, 2: 5, 4: 11, 8: 9, 0x10: 0x8000, 0x20: 10 }
	kit = GemRB.GetPlayerStat (pc, IE_KIT, 1)

	# the most common level-up paths - same class
	if GemRB.GetPlayerStat (pc, IDLUCommon.Levels[lucls-1]) > 0:
		if lukit == 0:
			# kitless class, matching current one(s)
			# not going to happen with vanilla data
			return
		else:
			# picked kit, matching current one(s)
			if lukit&kit:
				return
			else:
				raise RuntimeError("Picked second or non-existing kit (%d) of same class (%d)!?" %(lukit, lucls))
	else: # picked a new class
		kitBits = [((kit >> x) & 1)<<x  for x in range(31, -1, -1)]
		kitBits = [v for v in kitBits if v > 0]
		# check all the current kits - kit bits
		for k in kitBits:
			if k not in orderFavs:
				continue
			# picked a new kitless class
			if lukit == 0 and orderFavs[k] == lucls:
				return
			# picked a new kit
			if lukit > 0 and orderFavs[k] == lukit:
				return

	# only bad combinations remain
	# disable further level-ups in the base class
	specFlags = GemRB.GetPlayerStat (pc, IE_SPECFLAGS)
	if monkLevel:
		specFlags |= SPECF_MONKOFF
	if paladinLevel:
		specFlags |= SPECF_PALADINOFF
	GemRB.SetPlayerStat (pc, IE_SPECFLAGS, specFlags)

def OpenLUKitWindow ():
	global LUKitWindow

	# check if kit selection is needed at all
	# does the class have any kits at all?
	# if we're levelling up, we must also check we dont't already
	# have a kit with this class
	pc = GemRB.GameGetSelectedPCSingle ()
	LUClass = GemRB.GetVar ("LUClass") # index, not ID
	LUClassName = CommonTables.Classes.GetRowName (LUClass)
	LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
	hasKits = CommonTables.Classes.FindValue ("CLASS", LUClassID)
	kitIndex = GUICommonWindows.GetKitIndex (pc, LUClass)
	if hasKits is None or kitIndex > 0:
		kitName = CommonTables.Classes.GetRowName (kitIndex)
		kitID = CommonTables.Classes.GetValue (kitName, "ID", GTV_INT)
		if hasKits is None:
			kitID = 0
		HandleSpecFlagExclusion(pc, LUClassID, kitID)
		LUNextPress ()
		return

	LUKitWindow = Window = GemRB.LoadWindow (6, "GUIREC")

	# title
	Label = Window.GetControl (0x10000000)
	Label.SetText (9894)

	# next
	Button = Window.GetControl (0)
	Button.SetText (36789)
	Button.OnPress (LUNextPress)
	Button.SetState (IE_GUI_BUTTON_DISABLED)
	Button.MakeDefault ()

	# 1 does not exist, 2-10 are kit buttons
	# 11 scrollbar, 12 back
	Window.DeleteControl (12)

	# description
	TextArea = Window.GetControl (13)
	TextArea.SetText ("")

    # skip to the first kit of this class
	kitOffset = hasKits

	GemRB.SetVar ("LUKit", None)
	for i in range(9):
		Button = Window.GetControl (i+2)

		kitClassName = CommonTables.Classes.GetRowName (kitOffset+i)
		kitClass = CommonTables.Classes.GetValue (kitClassName, "CLASS", GTV_INT)
		if kitClass != LUClassID:
			Button.SetState (IE_GUI_BUTTON_DISABLED)
			continue

		kitTitle = CommonTables.Classes.GetRowName (kitOffset+i)
		kitTitle = CommonTables.Classes.GetValue (kitTitle, "NAME_REF", GTV_REF)
		Button.SetText (kitTitle)
		Button.SetState (IE_GUI_BUTTON_ENABLED)
		Button.SetVarAssoc ("LUKit", i)
		Button.OnPress (LUKitPress)
		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)

	Window.ShowModal (MODAL_SHADOW_NONE)

def LUKitPress ():
	Window = LUKitWindow
	Button = Window.GetControl (0)
	Button.SetState (IE_GUI_BUTTON_ENABLED)

	# display kit info
	TextArea = Window.GetControl (13)
	i = GemRB.GetVar ("LUKit") or 0
	LUClass = GemRB.GetVar ("LUClass") or 0
	LUClassName = CommonTables.Classes.GetRowName (LUClass)
	LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
	kitOffset = CommonTables.Classes.FindValue ("CLASS", LUClassID)
	kitName = CommonTables.Classes.GetRowName (i+kitOffset)
	kitDesc = CommonTables.Classes.GetValue (kitName, "DESC_REF", GTV_REF)
	TextArea.SetText (kitDesc)

	# set it to the kit value, so we don't need these gimnastics later
	kitID = CommonTables.Classes.GetValue (kitName, "ID", GTV_INT)
	GemRB.SetVar ("LUKit", kitID)
	GemRB.SetVar ("LUKitJustSet", LUClassID)

# continue with level up via chargen methods
def LUNextPress ():
	CloseLUWindow ()

	# pass on LevelDiff and selected class (already in LUClass)
	GemRB.SetVar ("LevelDiff", LevelDiff)

	# grant an ability point or three (each 4 levels)
	pc = GemRB.GameGetSelectedPCSingle ()

	if (GemRB.GetVar ("LUKitJustSet") or 0) != 0:
		oldKits = GemRB.GetPlayerStat (pc, IE_KIT, 1)
		kitID = GemRB.GetVar ("LUKit")
		GemRB.SetPlayerStat (pc, IE_KIT, oldKits | kitID)

		HandleSpecFlagExclusion(pc, GemRB.GetVar ("LUKitJustSet"), kitID)
		GemRB.SetVar ("LUKitJustSet", 0)

	levelSum = GemRB.GetPlayerStat (pc, IE_CLASSLEVELSUM)
	rankDiff = (levelSum+LevelDiff)//4 - levelSum//4
	if rankDiff > 0:
		import Abilities
		Abilities.OpenAbilitiesWindow (0, rankDiff)
	else:
		import Enemy
		Enemy.OpenEnemyWindow ()
	# both fire up the rest of the chain

def FinishLevelUp():
	# kit
	pc = GemRB.GameGetSelectedPCSingle ()
	LUKit = GemRB.GetVar ("LUKit") or 0

	# saving throws
	LUClass = GemRB.GetVar ("LUClass") or 0 # index, not ID
	LUClassName = CommonTables.Classes.GetRowName (LUClass)
	LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
	IDLUCommon.SetupSavingThrows (pc, LUClassID)

	# hit points
	Levels = [ GemRB.GetPlayerStat (pc, l) for l in IDLUCommon.Levels ]
	LevelDiff = GemRB.GetVar ("LevelDiff") or 0
	LevelDiffs = [ 0 ] * len(Levels)
	LevelDiffs[LUClass] = LevelDiff
	# SetupHP expects the target level already
	Levels[LUClass] += LevelDiff
	LUCommon.SetupHP (pc, Levels, LevelDiffs)

	# add class/kit resistances iff we chose a new class
	levelStat = IDLUCommon.Levels[LUClass]
	oldLevel = GemRB.GetPlayerStat(pc, levelStat, 1)
	# use the kit name if it is available
	LUKitName = LUClassName
	if LUKit != 0:
		kitIndex = CommonTables.Classes.FindValue ("ID", LUKit)
		LUKitName = CommonTables.Classes.GetRowName (kitIndex)
	if oldLevel == 0:
		IDLUCommon.AddResistances (pc, LUKitName, "clssrsmd")

	# bab (to hit)
	BABTable = CommonTables.Classes.GetValue (LUClassName, "TOHIT")
	BABTable = GemRB.LoadTable (BABTable)
	currentBAB = GemRB.GetPlayerStat (pc, IE_TOHIT, 1)
	oldBAB = BABTable.GetValue (str(oldLevel), "BASE_ATTACK", GTV_INT)
	newLevel = oldLevel + LevelDiff
	newBAB = BABTable.GetValue (str(newLevel), "BASE_ATTACK", GTV_INT)
	GemRB.SetPlayerStat (pc, IE_TOHIT, currentBAB + newBAB - oldBAB)

	# class level
	GemRB.SetPlayerStat (pc, levelStat, newLevel)

	# now we're finally done
	GemRB.SetVar ("LevelDiff", 0)
	GemRB.SetVar ("LUClass", None)
	GemRB.SetVar ("LUKit", 0)

	UpdateRecordsWindow (RecordsWindow)


###################################################
# End of file GUIREC.py



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, name (GUICG5)
import GemRB
from GUIDefines import *

import CharGenCommon
import GUICommon


NameWindow = 0
NameField = 0
DoneButton = 0

def OnLoad():
	global NameWindow, NameField, DoneButton
	
	NameWindow = GemRB.LoadWindow(5, "GUICG")

	BackButton = NameWindow.GetControl(3)
	BackButton.SetText(15416)

	DoneButton = NameWindow.GetControl(0)
	DoneButton.SetText(11973)
	DoneButton.MakeDefault()
	DoneButton.SetState(IE_GUI_BUTTON_DISABLED)

	NameField = NameWindow.GetControl(2)
	NameField.SetText (GemRB.GetToken ("CHARNAME"))
	EditChange ()

	DoneButton.OnPress (NextPress)
	BackButton.OnPress (lambda: CharGenCommon.back(NameWindow))
	NameField.OnChange (EditChange)
	NameWindow.ShowModal(MODAL_SHADOW_GRAY)
	NameField.Focus()
	return

def NextPress():
	NameWindow.Close()
	Name = NameField.QueryText()
	#check length?
	#seems like a good idea to store it here for the time being
	MyChar = GemRB.GetVar ("Slot")
	GemRB.SetPlayerName (MyChar, Name, 0)
	GemRB.SetToken ("CHARNAME", Name)
	CharGenCommon.next()
	return

def EditChange():
	Name = NameField.QueryText()
	if len(Name)==0:
		DoneButton.SetState(IE_GUI_BUTTON_DISABLED)
	else:
		DoneButton.SetState(IE_GUI_BUTTON_ENABLED)
	return



#!/usr/bin/python
# GemRB - Infinity Engine Emulator
# Copyright (C) 2011 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
import sys

def usage(msg):
  print("Error:", msg)
  print("\nUsage:", sys.argv[0], 'filename APPEND|APPEND_COL "$|value1  [$|value2] ... [$|valueN]" [-r]\n')
  print("Passing $ will result in an empty cell. Use it for the first two entries when appending columns,\nso you don't break the 2da signature or default value. Passing -r at the end will right-align cells.")
  print()
  print("Example:")
  print("python extend2da.py gemrb/override/bg1/classes.2da APPEND 'HACKER 1 3 0 0 0 0 0 0 0 0 0 0 0 0 0 0'")

# get the longest row for comparison, while caching the reads
def readAndGetMaxLength(f):
  global lines

  maxL = i = 0
  for line in f:
    lines.append(line.rstrip())
    if len(lines[i]) > maxL and chr(line[0]) != "#":
      maxL = len(lines[i])
    i = i + 1
  return maxL

def appendCol(f, max):
  global lines, data

  f.seek(0)
  f.truncate()
  i = 0
  for cell in data:
    if i == 2:
      colName = cell

    if cell == "$" or lines[i] == b"" or chr(lines[i][0]) == "#":
      f.write(lines[i] + b"\n")
      i = i + 1
      continue

    padding = max - len(lines[i]) + 2 # 2 spaces as field separator
    if i > 2 and "-r" in flags:
      padding = padding  + (len(colName) - len(cell)) # right-align
    f.write(lines[i] + (" "*padding + cell + "\n").encode('ascii'))

    i = i + 1

def CheckCountsMismatch(mode):
  global data, lines

  if mode == "APPEND_COL":
    msg = "rows"
    rc = 13
    count = len(lines)
  elif mode == "APPEND":
    msg = "columns"
    rc = 14
    count = len(lines[3].split()) # skip the 2da header
  else:
    return

  mismatch = count - len(data)
  if mismatch:
    print ("Error: you need to specify the data for all the %s!" % msg)
    print ("Table has %d, while you provided %d!" % (count, len(data)))
    sys.exit(rc)


def appendRow(f):
  global data

  # get a line with real content for measurement
  line = lines[3]
  vals = line.split()

  f.seek(0, 2) # seek to the end
  i = 0
  for cell in data:
    # width of cell (content + padding) from first line
    cw = len(line)-len(line.lstrip(vals[i]).lstrip())
    line = line[cw:]
    padding = cw - len(cell)
    if padding < 1:
      # ideally we would reformat the whole table, but meh
      padding = 2
    padding = " " * padding
    f.write((("%s"+padding) % cell).encode('ascii'))
    i += 1
  # remove the extraneus ending padding
  f.seek(f.tell()-2)
  f.truncate()
  f.write(b"\n")

################# MAIN ####################################
if len(sys.argv) < 4:
  usage("missing parameters")
  sys.exit(11)

filename = sys.argv[1]
mode = sys.argv[2].upper() # APPEND / APPEND_COL
data = sys.argv[3:][0].split()
flags = sys.argv[4] if len(sys.argv) > 4 else ""

if mode != "APPEND" and mode != "APPEND_COL":
  usage("invalid mode parameter")
  sys.exit(12)

lines = []

with open(filename, 'r+b') as f:
  max = readAndGetMaxLength(f)
  CheckCountsMismatch(mode)
  if mode == "APPEND_COL":
    appendCol(f, max)
  elif mode == "APPEND":
    appendRow(f)

# tests
# python admin/extend2da.py gemrb/unhardcoded/bg1/classes.2da APPEND_COL "$ $ BOGOSITY 0 0 0 00 0 0 0 0 0 0 0 0 0 0 2 3 4 7"
# python admin/extend2da.py gemrb/unhardcoded/bg1/classes.2da APPEND "HACKER 1 3 0 0 0 00 0 0 0 0 0 0 0 0 0 0 0 0"




# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# MessageWindow.py - scripts and GUI for main (walk) window

###################################################

import GemRB
import ActionsWindow as ActionsWindowModule
import GUIClasses
import GUICommon
import GUICommonWindows
import CommonWindow
import GUIWORLD
import Clock
import PortraitWindow
from GameCheck import MAX_PARTY_SIZE
from GUIDefines import *

MWindow = 0

def OnLoad():
	global MWindow

	# TODO: we can uncomment the "HIDE_CUT" lines below to hide the windows for cutscenes
	# the original doesn't hide them and it looks like there is a map drawing bug at the bottom of the screen due to the bottom
	# row of tiles getting squished for not fitting perfectly on screen (tho I havent seen this in BG2, but maybe wasnt paying attention)

	ActionsWindow = GemRB.LoadWindow(0, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_LEFT)
	ActionsWindow.AddAlias("ACTWIN")
	#ActionsWindow.AddAlias("HIDE_CUT", 1)
	ActionsWindow.AddAlias("NOT_DLG", 0)
	ActionsWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)

	OptionsWindow = GemRB.LoadWindow(2, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_RIGHT)
	OptionsWindow.AddAlias("OPTWIN")
	#OptionsWindow.AddAlias("HIDE_CUT", 2)
	OptionsWindow.AddAlias("NOT_DLG", 1)
	OptionsWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	GemRB.SetVar ("PortraitPosition", 1)

	MWindow = GemRB.LoadWindow(7, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)
	MWindow.SetFlags(WF_DESTROY_ON_CLOSE, OP_NAND)
	MWindow.AddAlias("MSGWIN")
	MWindow.AddAlias("HIDE_CUT", 0)
	MWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)

	PortraitWin = PortraitWindow.OpenPortraitWindow (WINDOW_BOTTOM|WINDOW_HCENTER)
	PortraitWin.AddAlias("NOT_DLG", 2)
	PortraitWin.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)

	pframe = PortraitWin.GetFrame()
	pframe['x'] -= 16
	PortraitWin.SetFrame(pframe)

	MessageTA = MWindow.GetControl (1)
	MessageTA.SetFlags (IE_GUI_TEXTAREA_AUTOSCROLL|IE_GUI_TEXTAREA_HISTORY)
	MessageTA.SetResizeFlags(IE_GUI_VIEW_RESIZE_ALL)
	MessageTA.AddAlias("MsgSys", 0)
	MessageTA.SetColor(ColorRed, TA_COLOR_OPTIONS)
	MessageTA.SetColor(ColorWhite, TA_COLOR_HOVER)

	CloseButton = MWindow.GetControl (0)
	CloseButton.SetText(28082)
	CloseButton.OnPress (MWindow.Close)
	CloseButton.MakeDefault()
	
	OpenButton = OptionsWindow.GetControl (10)
	OpenButton.OnPress (MWindow.Focus)

	SetupClockWindowControls (ActionsWindow)
	GUICommonWindows.SetupMenuWindowControls (OptionsWindow)

	UpdateControlStatus ()
	
def SetupClockWindowControls (Window):
	# time button
	Button = Window.GetControl (0)
	Clock.CreateClockButton(Button)

	# 41627 - Return to the Game World
	Button = Window.GetControl (2)
	Button.OnPress (GUICommonWindows.CloseTopWindow)
	Button.SetTooltip (41627)

	# Select all characters
	Button = Window.GetControl (1)
	Button.SetTooltip (41659)
	Button.OnPress (GUICommon.SelectAllOnPress)

	# Abort current action
	Button = Window.GetControl (3)
	Button.SetTooltip (41655)
	Button.OnPress (ActionsWindowModule.ActionStopPressed)

	# Formations
	import GUIWORLD
	Button = Window.GetControl (4)
	Button.SetTooltip (44945)
	Button.OnPress (GUIWORLD.OpenFormationWindow)

	return

def UpdateControlStatus ():
	if GemRB.GetGUIFlags() & (GS_DIALOGMASK|GS_DIALOG):
		Label = MWindow.GetControl (0x10000003)
		Label.SetText (str (GemRB.GameGetPartyGold ()))

		MWindow.Focus()
	elif MWindow:
		MWindow.Close()




# !!! Keep these synchronized with maze definitions in Game.h !!!


#maze header indices
MH_POS1X = 0
MH_POS1Y = 1
MH_POS2X = 2
MH_POS2Y = 3
MH_POS3X = 4
MH_POS3Y = 5
MH_POS4X = 6
MH_POS4Y = 7
MH_TRAPCOUNT = 8
MH_INITED = 9
MH_UNKNOWN28 = 10
MH_UNKNOWN2C = 11

#maze entry indices
ME_OVERRIDE = 0
ME_VALID = 1
ME_ACCESSIBLE = 2
ME_TRAP = 3
ME_WALLS = 4
ME_VISITED = 5

# Wall directions
WALL_SOUTH = 1
WALL_NORTH = 2
WALL_EAST = 4
WALL_WEST = 8

#maximum maze entry
MAZE_ENTRY_COUNT = 64
MAZE_MAX_DIM = 8



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, import (GUICG20)
import GemRB

import CharOverview
from GUIDefines import *

#import from a character sheet
MainWindow = 0
PortraitButton = 0
ImportWindow = 0
TextAreaControl = 0
DoneButton = 0

def OnLoad():
	global MainWindow, PortraitButton
	global ImportWindow, TextAreaControl, DoneButton

	MainWindow = GemRB.LoadWindow(0, "GUICG")

	PortraitButton = MainWindow.GetControl (12)
	PortraitButton.SetFlags(IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_NO_IMAGE,OP_SET)

	ImportWindow = GemRB.LoadWindow(20, "GUICG")
	CharOverview.PositionCharGenWin (ImportWindow)

	TextAreaControl = ImportWindow.GetControl(4)
	TextAreaControl.SetText(10963)

	TextAreaControl = ImportWindow.GetControl(2)
	TextAreaControl.ListResources(CHR_EXPORTS)

	DoneButton = ImportWindow.GetControl(0)
	DoneButton.SetText(36789)
	DoneButton.SetState(IE_GUI_BUTTON_DISABLED)

	CancelButton = ImportWindow.GetControl(1)
	CancelButton.SetText(15416)
	CancelButton.MakeEscape()
	
	# disable the three extraneous buttons in the bottom row
	for i in [16, 13, 15]:
		TmpButton = MainWindow.GetControl(i)
		TmpButton.SetState(IE_GUI_BUTTON_DISABLED)

	DoneButton.OnPress (DonePress)
	CancelButton.OnPress (CancelPress)
	TextAreaControl.OnSelect (SelectFile)
	ImportWindow.Focus()
	return

def DonePress():
	if ImportWindow:
		ImportWindow.Close ()
	if MainWindow:
		MainWindow.Close ()
	#this part is fuzzy
	#we don't have the character as an object in the chargen
	#but we just imported a complete object
	#either we take the important stats and destroy the object
	#or start with an object from the beginning
	#or use a different script here???
	GemRB.SetVar ("ImportedChar", 1)
	GemRB.SetNextScript("CharGen7")
	return
	
def CancelPress():
	if ImportWindow:
		ImportWindow.Close ()
	if MainWindow:
		MainWindow.Close ()
	GemRB.SetNextScript(GemRB.GetToken("NextScript"))
	return

def SelectFile():
	FileName = TextAreaControl.QueryText()
	Slot = GemRB.GetVar("Slot")
	GemRB.CreatePlayer(FileName, Slot| 0x8000, 1)
	GemRB.SetToken ("CHARNAME", GemRB.GetPlayerName (Slot))
	Portrait = GemRB.GetPlayerPortrait (Slot, 0)
	GemRB.SetToken ("SmallPortrait", GemRB.GetPlayerPortrait (Slot, 1)["ResRef"])
	GemRB.SetToken ("LargePortrait", Portrait["ResRef"])

	PortraitButton.SetPicture (Portrait["Sprite"], "NOPORTLG")
	ImportWindow.Focus() #bring it to the front
	DoneButton.SetState(IE_GUI_BUTTON_ENABLED)
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# ie_stats.py - definitions of creature stats codes

# !!! NOTE: Keep this file synchronized with gemrb/includes/ie_stats.h !!!

# EA values
INANIMATE     = 1
PC            = 2
FAMILIAR      = 3
ALLY          = 4
CONTROLLED    = 5
#charmed is 7 inside the engine???
CHARMED       = 7
GOODBUTRED    = 28
GOODBUTBLUE   = 29
GOODCUTOFF    = 30
NOTGOOD       = 31
ANYTHING      = 126
NEUTRAL       = 128
NOTEVIL       = 199
EVILCUTOFF    = 200
EVILBUTGREEN  = 201
EVILBUTBLUE   = 202
CHARMEDPC     = 254
ENEMY         = 255

# state bits (IE_STATE)
STATE_BERSERK = 2
STATE_PANIC = 4
STATE_HELPLESS = 1 + 32
STATE_PETRIFIED = 8 + 64 + 128
STATE_DEAD    = 2048
STATE_POISONED = 0x4000

MC_WAS_FIGHTER = 0x0008
MC_WAS_MAGE = 0x0010
MC_WAS_CLERIC = 0x0020
MC_WAS_THIEF = 0x0040
MC_WAS_DRUID = 0x0080
MC_WAS_RANGER = 0x0100
MC_WAS_ANY_CLASS = MC_WAS_FIGHTER|MC_WAS_MAGE|MC_WAS_CLERIC|MC_WAS_THIEF|MC_WAS_DRUID|MC_WAS_RANGER

MC_FALLEN_PALADIN = 0x200
MC_FALLEN_RANGER = 0x400
MC_EXPORTABLE = 0x800

MC_PLOT_CRITICAL = 0x2000
MC_BEENINPARTY = 0x8000
MC_SEENPARTY = 0x10000

# specflag bits
SPECF_DRIVEN = 1
SPECF_CRITIMMUNITY = 2
SPECF_PALADINOFF = 4
SPECF_MONKOFF = 8

# stats
IE_HITPOINTS 			= 0
IE_MAXHITPOINTS			= 1
IE_ARMORCLASS 			= 2
IE_ACCRUSHINGMOD 		= 3
IE_ACMISSILEMOD 		= 4
IE_ACPIERCINGMOD 		= 5
IE_ACSLASHINGMOD 		= 6
IE_TOHIT 			= 7
IE_NUMBEROFATTACKS 		= 8
IE_SAVEVSDEATH 			= 9
IE_SAVEVSWANDS 			= 10
IE_SAVEVSPOLY 			= 11
IE_SAVEVSBREATH 		= 12
IE_SAVEVSSPELL 			= 13
IE_SAVEFORTITUDE		= 9
IE_SAVEREFLEX 			= 10
IE_SAVEWILL 			= 11
IE_RESISTFIRE 			= 14
IE_RESISTCOLD 			= 15
IE_RESISTELECTRICITY 		= 16
IE_RESISTACID 			= 17
IE_RESISTMAGIC 			= 18
IE_RESISTMAGICFIRE 		= 19
IE_RESISTMAGICCOLD 		= 20
IE_RESISTSLASHING 		= 21
IE_RESISTCRUSHING 		= 22
IE_RESISTPIERCING 		= 23
IE_RESISTMISSILE 		= 24
IE_LORE 			= 25
IE_LOCKPICKING 			= 26
IE_STEALTH 			= 27
IE_TRAPS 			= 28
IE_PICKPOCKET 			= 29
IE_FATIGUE 			= 30
IE_INTOXICATION 		= 31
IE_LUCK 			= 32
IE_TRACKING 			= 33
IE_LEVEL 			= 34
IE_LEVELFIGHTER		= 34 # pst, iwd2
IE_SEX 				= 35
IE_STR 				= 36
IE_STREXTRA 			= 37
IE_INT 				= 38
IE_WIS 				= 39
IE_DEX 				= 40
IE_CON 				= 41
IE_CHR 				= 42
IE_XPVALUE 			= 43
IE_XP 				= 44
IE_GOLD 			= 45
IE_MORALEBREAK 			= 46
IE_MORALERECOVERYTIME 		= 47
IE_REPUTATION 			= 48
IE_HATEDRACE 			= 49
IE_DAMAGEBONUS 			= 50
IE_SPELLFAILUREMAGE 		= 51
IE_SPELLFAILUREPRIEST 		= 52
IE_SPELLDURATIONMODMAGE 	= 53
IE_SPELLDURATIONMODPRIEST 	= 54
IE_TURNUNDEADLEVEL 		= 55
IE_BACKSTABDAMAGEMULTIPLIER 	= 56
IE_LAYONHANDSAMOUNT 		= 57
IE_HELD 			= 58
IE_POLYMORPHED 			= 59
IE_TRANSLUCENT 			= 60
IE_IDENTIFYMODE 		= 61
IE_ENTANGLE 			= 62
IE_SANCTUARY 			= 63
IE_MINORGLOBE 			= 64
IE_SHIELDGLOBE 			= 65
IE_GREASE 			= 66
IE_WEB 				= 67
IE_LEVEL2 			= 68
IE_LEVELMAGE		= 68 # pst, iwd2
IE_LEVEL3 			= 69
IE_LEVELTHIEF		= 69 # pst, iwd2
IE_CASTERHOLD 			= 70
IE_ENCUMBRANCE                  = 71
IE_MISSILEHITBONUS 		= 72
IE_MAGICDAMAGERESISTANCE 	= 73
IE_RESISTPOISON 		= 74
IE_DONOTJUMP 			= 75
IE_AURACLEANSING 		= 76
IE_MENTALSPEED 			= 77
IE_PHYSICALSPEED 		= 78
IE_CASTINGLEVELBONUSMAGE 	= 79
IE_CASTINGLEVELBONUSCLERIC 	= 80
IE_SEEINVISIBLE 		= 81
IE_IGNOREDIALOGPAUSE 		= 82
IE_MINHITPOINTS 		= 83
IE_HITBONUSRIGHT 		= 84
IE_HITBONUSLEFT 		= 85
IE_DAMAGEBONUSRIGHT 		= 86
IE_DAMAGEBONUSLEFT 		= 87
IE_STONESKINS 			= 88
IE_FEAT_BOW                     = 89
IE_FEAT_CROSSBOW                = 90
IE_FEAT_SLING                   = 91
IE_FEAT_AXE                     = 92
IE_FEAT_MACE                    = 93
IE_FEAT_FLAIL                   = 94
IE_FEAT_POLEARM                 = 95
IE_FEAT_HAMMER                  = 96
IE_FEAT_STAFF                   = 97
IE_FEAT_GREAT_SWORD             = 98
IE_FEAT_LARGE_SWORD             = 99
IE_FEAT_SMALL_SWORD             = 100
IE_FEAT_TOUGHNESS               = 101
IE_FEAT_ARMORED_ARCANA          = 102
IE_FEAT_CLEAVE                  = 103
IE_FEAT_ARMOUR                  = 104
IE_FEAT_ENCHANTMENT             = 105
IE_FEAT_EVOCATION               = 106
IE_FEAT_NECROMANCY              = 107
IE_FEAT_TRANSMUTATION           = 108
IE_FEAT_SPELL_PENETRATION       = 109
IE_FEAT_EXTRA_RAGE              = 110
IE_FEAT_EXTRA_SHAPE             = 111
IE_FEAT_EXTRA_SMITING           = 112
IE_FEAT_EXTRA_TURNING           = 113
IE_FEAT_BASTARDSWORD            = 114
IE_ALCHEMY                      = 115
IE_ANIMALS                      = 116
IE_BLUFF                        = 117
IE_CONCENTRATION                = 118
IE_DIPLOMACY                    = 119
IE_INTIMIDATE                   = 120
IE_SEARCH                       = 121
IE_SPELLCRAFT                   = 122
IE_MAGICDEVICE                  = 123
IE_SPECFLAGS                    = 124
IE_PROFICIENCYBASTARDSWORD 	= 89
IE_PROFICIENCYLONGSWORD 	= 90
IE_PROFICIENCYSHORTSWORD 	= 91
IE_PROFICIENCYAXE 		= 92			
IE_PROFICIENCYTWOHANDEDSWORD 	= 93
IE_PROFICIENCYKATANA 		= 94
IE_PROFICIENCYSCIMITARWAKISASHININJATO = 95
IE_PROFICIENCYDAGGER 		= 96
IE_PROFICIENCYWARHAMMER 	= 97
IE_PROFICIENCYSPEAR 		= 98
IE_PROFICIENCYHALBERD 		= 99			
IE_PROFICIENCYFLAILMORNINGSTAR 	= 100
IE_PROFICIENCYMACE 		= 101			
IE_PROFICIENCYQUARTERSTAFF 	= 102		
IE_PROFICIENCYCROSSBOW 		= 103		
IE_PROFICIENCYLONGBOW 		= 104		
IE_PROFICIENCYSHORTBOW 		= 105		
IE_PROFICIENCYDART 		= 106
IE_PROFICIENCYSLING 		= 107	
IE_PROFICIENCYBLACKJACK 	= 108	
IE_PROFICIENCYGUN 		= 109		
IE_PROFICIENCYMARTIALARTS 	= 110	
IE_PROFICIENCY2HANDED 		= 111 
IE_PROFICIENCYSWORDANDSHIELD 	= 112
IE_PROFICIENCYSINGLEWEAPON 	= 113	
IE_PROFICIENCY2WEAPON 		= 114  			
IE_PROFICIENCYCLUB		= 115
IE_EXTRAPROFICIENCY2 		= 116
IE_EXTRAPROFICIENCY3 		= 117
IE_EXTRAPROFICIENCY4 		= 118
IE_EXTRAPROFICIENCY5 		= 119
IE_EXTRAPROFICIENCY6 		= 120
IE_EXTRAPROFICIENCY7 		= 121
IE_EXTRAPROFICIENCY8 		= 122
IE_EXTRAPROFICIENCY9 		= 123
IE_EXTRAPROFICIENCY10 		= 124
IE_EXTRAPROFICIENCY11 		= 125
IE_EXTRAPROFICIENCY12 		= 126
IE_EXTRAPROFICIENCY13 		= 127
IE_EXTRAPROFICIENCY14 		= 128
IE_EXTRAPROFICIENCY15 		= 129
IE_EXTRAPROFICIENCY16 		= 130
IE_EXTRAPROFICIENCY17 		= 131
IE_FEATS1                   = 131
IE_EXTRAPROFICIENCY18 		= 132
IE_FEATS2                   = 132
IE_EXTRAPROFICIENCY19 		= 133
IE_FEATS3                   = 133
IE_EXTRAPROFICIENCY20 		= 134
IE_FREESLOTS			= 134 #not an error (PST)
IE_HIDEINSHADOWS		= 135
IE_DETECTILLUSIONS  		= 136
IE_SETTRAPS 			= 137
IE_PUPPETMASTERID   		= 138
IE_PUPPETMASTERTYPE 		= 139
IE_PUPPETTYPE   		= 140
IE_PUPPETID 			= 141
IE_CHECKFORBERSERK  		= 142
IE_BERSERKSTAGE1		= 143
IE_BERSERKSTAGE2		= 144
IE_DAMAGELUCK   		= 145
IE_CRITICALHITBONUS 		= 146
IE_VISUALRANGE  		= 147
IE_EXPLORE  			= 148
IE_THRULLCHARM  		= 149
IE_SUMMONDISABLE		= 150
IE_HITBONUS 			= 151    
IE_KIT  			= 152 
IE_FORCESURGE   		= 153
IE_SURGEMOD 			= 154
IE_IMPROVEDHASTE		= 155
IE_SCRIPTINGSTATE1  		= 156
IE_SCRIPTINGSTATE2  		= 157
IE_SCRIPTINGSTATE3  		= 158
IE_SCRIPTINGSTATE4  		= 159
IE_SCRIPTINGSTATE5  		= 160
IE_SCRIPTINGSTATE6  		= 161
IE_SCRIPTINGSTATE7  		= 162   
IE_SCRIPTINGSTATE8  		= 163
IE_SCRIPTINGSTATE9  		= 164
IE_SCRIPTINGSTATE10 		= 165
IE_MELEETOHIT			= 166
IE_MELEEDAMAGE			= 167
IE_MISSILEDAMAGE		= 168
IE_NOCIRCLE			= 169
IE_FISTHIT			= 170
IE_FISTDAMAGE			= 171
IE_TITLE1			= 172
IE_TITLE2			= 173
IE_DISABLEOVERLAY		= 174
IE_DISABLEBACKSTAB		= 175
#the next two are overwritten by us in PST
IE_XP_MAGE	 		= 176 # In PST this stores secondary level exp
IE_XP_THIEF	 		= 177 # In PST this stores tertiary level exp
#but in other games they're still used sometimes
IE_SCRIPTINGSTATE21  		= 176
IE_SCRIPTINGSTATE22  		= 177
IE_SCRIPTINGSTATE23  		= 178
IE_SCRIPTINGSTATE24  		= 179
IE_SCRIPTINGSTATE25  		= 180
IE_SCRIPTINGSTATE26  		= 181
IE_SCRIPTINGSTATE27  		= 182
#these are original
IE_ENABLEOFFSCREENAI            = 183
IE_EXISTANCEDELAY               = 184
IE_ATTACKNUMBERDOUBLE           = 185
IE_DISABLECHUNKING              = 186
IE_NOTURNABLE                   = 187
#188 was summondisable2 in original
IE_STONESKINSGOLEM		= 199
IE_LEVELDRAIN			= 200
IE_AVATARREMOVAL                = 201
#202 is unused
# GemRB Specific Defines
IE_IMMUNITY                     = 203
IE_DISABLEDBUTTON		= 204
IE_ANIMATION_ID			= 205
IE_STATE_ID			= 206
IE_EXTSTATE_ID                  = 207
IE_METAL_COLOR			= 208
IE_MINOR_COLOR			= 209
IE_MAJOR_COLOR			= 210
IE_SKIN_COLOR			= 211
IE_LEATHER_COLOR		= 212
IE_ARMOR_COLOR			= 213
IE_HAIR_COLOR			= 214
IE_MC_FLAGS			= 215
IE_CLASSLEVELSUM		= 216
IE_ALIGNMENT			= 217
IE_CASTING			= 218
IE_ARMOR_TYPE			= 219
IE_TEAM				= 220
IE_FACTION			= 221
IE_SUBRACE			= 222
IE_UNUSED_SKILLPTS  = 223
IE_SPECIES                      = 223 #pst specific
IE_HATEDRACE2                   = 224
IE_HATEDRACE3                   = 225
IE_HATEDRACE4                   = 226
IE_HATEDRACE5                   = 227
IE_HATEDRACE6                   = 228
IE_HATEDRACE7                   = 229
IE_HATEDRACE8                   = 230
# These are in original PST, IWD, IWD2, but not as stats
IE_RACE				= 231
IE_CLASS			= 232
IE_GENERAL			= 233
IE_EA				= 234
IE_SPECIFIC			= 235
IE_SAVEDXPOS                    = 236
IE_SAVEDYPOS                    = 237
IE_SAVEDFACE                    = 238
#239 user defined stat
IE_LEVELBARBARIAN               = 240
IE_LEVELBARD                    = 241
IE_LEVELCLERIC                  = 242
IE_LEVELDRUID                   = 243
IE_LEVELMONK                    = 244
IE_LEVELPALADIN                 = 245
IE_LEVELRANGER                  = 246
IE_LEVELSORCERER                = 247
IE_LEVELSHAMAN                  = 248
#249 IE_LEVELCLASS13
#the remaining stats are for our internal use now
IE_DIALOGRANGE                  = 250 # distance for dialogue
IE_MOVEMENTRATE                 = 251
IE_MORALE                       = 252
IE_BOUNCE                       = 253
IE_MIRRORIMAGES                 = 254

#these stats exist only in PC's (but we access only PCs anyway)
IE_EXPERTISE                    = 0x1003
IE_POWERATTACK                  = 0x1004
IE_ARTERIAL_STRIKE              = 0x1005
IE_HAMSTRING                    = 0x1006
IE_RAPID_SHOT                   = 0x1007

# internal class indices
# Keep these synchronized with Actor.h
ISFIGHTER = 0
ISMAGE = 1
ISTHIEF = 2
ISBARBARIAN = 3
ISBARD = 4
ISCLERIC = 5
ISDRUID = 6
ISMONK = 7
ISPALADIN = 8
ISRANGER = 9
ISSORCERER = 10
ISSHAMAN = 11
ISCLASS13 = 12

# End of file ie_stats.py



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import GemRB
import GameCheck
import GUICommon
import Spellbook
import CommonTables
from GUIDefines import *
from ie_stats import *

# storage variables
pc = 0
chargen = 0
KitMask = 0
Class = 0

# basic spell selection
SpellsWindow = 0		# << spell selection window
DoneButton = 0			# << done/next button
SpellsTextArea = 0		# << spell description area
SpellsSelectPointsLeft = [0]*9	# << spell selections left per level
Spells = [0]*9			# << spells learnable per level
SpellTopIndex = 0		# << scroll bar index
SpellBook = []			# << array containing all the spell indexes to learn
ButtonCount = 24		# number of spell slots in the window
MemoBook = [0]*ButtonCount			# array containing all the spell indexes to memorize
Memorization = 0			# marker for the memorisation part
SpellLevel = 0			# << current level of spells
SpellStart = 0			# << starting id of the spell list
SpellPointsLeftLabel = 0	# << label indicating the number of points left
EnhanceGUI = 0			# << scrollbars and extra spell slot for sorcs on LU
BonusPoints = [0]*9		# bonus learning/memo points

# chargen only
SpellsPickButton = None	# << button to select random spells
SpellsCancelButton = 0		# << cancel chargen

IWD2 = False
SpellBookType = IE_SPELL_TYPE_WIZARD
if GameCheck.IsIWD2():
	IWD2 = True

NewScrollBarID = 1000

def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, booktype=0):
	"""Opens the spells selection window.

	table should refer to the name of the classes MXSPLxxx.2da.
	level contains the current level of the actor.
	diff contains the difference from the old level.
	kit should always be GetKitIndex except when dualclassing.
	gen is true if this is for character generation.
	recommend is used in bg2 for spell recommendation / autopick."""

	global SpellsWindow, DoneButton, SpellsSelectPointsLeft, Spells, chargen, SpellPointsLeftLabel
	global SpellsTextArea, SpellTopIndex, SpellBook, SpellLevel, pc, SpellStart, BonusPoints
	global KitMask, EnhanceGUI, Memorization, SpellBookType, SpellsPickButton, ButtonCount, Class

	#enhance GUI?
	if (GemRB.GetVar("GUIEnhancements")&GE_SCROLLBARS) and not IWD2:
		EnhanceGUI = 1

	# save our pc
	pc = actor
	chargen = gen

	# this ensures compatibility with chargen, sorc, and dual-classing
	if kit == 0:
		KitMask = 0x4000
	else: # need to implement this if converted to CharGen
		KitMask = kit

	if IWD2:
		# save the spellbook type (class) that corresponds to our table
		SpellBookType = booktype
		if not chargen:
			ButtonCount = 30

	# make sure there is an entry at the given level (bard)
	SpellLearnTable = table
	SpellsToMemoTable = GemRB.LoadTable (table)
	if not SpellsToMemoTable.GetValue (str(level), str(1), GTV_INT):
		if chargen:
			if GameCheck.IsBG2OrEE ():
				GemRB.SetNextScript("GUICG6")
			elif GameCheck.IsBG1():
				# HACK
				from CharGenCommon import next
				next()
			elif IWD2:
				GemRB.SetNextScript ("CharGen7")
		return

	# load our window
	if chargen:
		SpellsWindow = GemRB.LoadWindow (7, "GUICG")

		if GameCheck.IsBG2OrEE ():
			import CharGenCommon
			CharGenCommon.PositionCharGenWin (SpellsWindow)
		elif IWD2:
			import CharOverview
			CharOverview.PositionCharGenWin (SpellsWindow)

		DoneButton = SpellsWindow.GetControl (0)
		SpellsTextArea = SpellsWindow.GetControl (27)
		SpellPointsLeftLabel = SpellsWindow.GetControl (0x1000001b)
		if (EnhanceGUI):
			sb = SpellsWindow.CreateScrollBar (NewScrollBarID, {'x' : 325, 'y' : 42, 'w' : 16, 'h' : 252})
			sb.SetVisible(False)
		SpellStart = 2

		# cancel button only applicable for chargen
		SpellsCancelButton = SpellsWindow.GetControl(29)
		SpellsCancelButton.SetState(IE_GUI_BUTTON_ENABLED)
		SpellsCancelButton.OnPress (SpellsCancelPress)
		SpellsCancelButton.SetText(13727)
		SpellsCancelButton.MakeEscape()

		if (recommend):
			# recommended spell picks
			SpellsPickButton = SpellsWindow.GetControl(30)
			SpellsPickButton.SetState(IE_GUI_BUTTON_ENABLED)
			SpellsPickButton.OnPress (SpellsPickPress)
			SpellsPickButton.SetText(34210)
	else:
		SpellsWindow = GemRB.LoadWindow (8, "GUIREC")
		if IWD2:
			DoneButton = SpellsWindow.GetControl (33)
			SpellsTextArea = SpellsWindow.GetControl(30)
			SpellPointsLeftLabel = SpellsWindow.GetControl (0x10000022)
		else:
			DoneButton = SpellsWindow.GetControl (28)
			SpellsTextArea = SpellsWindow.GetControl(26)
			SpellPointsLeftLabel = SpellsWindow.GetControl (0x10000018)
		if(EnhanceGUI):
			sb = SpellsWindow.CreateScrollBar (NewScrollBarID, {'x' : 290, 'y' : 142, 'w' : 16, 'h' : 252})
			sb.SetVisible(False)
			#25th spell button for sorcerers
			SpellsWindow.CreateButton (24, 231, 345, 42, 42)

		SpellStart = 0

	# setup our variables
	GemRB.SetVar ("SpellTopIndex", 0)
	Memorization = 0
	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
	if IWD2 and not chargen:
		LUClass = GemRB.GetVar ("LUClass") or 0
		LUClassName = CommonTables.Classes.GetRowName (LUClass)
		LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
		Class = LUClassID

	# the done button also doubles as a next button
	DoneButton.SetDisabled(True)
	DoneButton.OnPress (SpellsDonePress)
	DoneButton.SetText(11973)
	DoneButton.MakeDefault()

	# adjust the table for the amount of spells available for learning for free
	# bg2 had SPLSRCKN, iwd2 also SPLBRDKN, but all the others lacked the tables
	if SpellLearnTable == "MXSPLSOR" or SpellLearnTable == "MXSPLSRC":
		SpellLearnTable = "SPLSRCKN"
	elif SpellLearnTable == "MXSPLBRD":
		SpellLearnTable = "SPLBRDKN"
	elif SpellLearnTable == "MXSPLSHM":
		SpellLearnTable = "SPLSHMKN"
	# ... which is also important for mages during chargen and then never again
	elif SpellLearnTable == "MXSPLWIZ":
		SpellLearnTable = "SPLWIZKN"
	else:
		print("OpenSpellsWindow: unhandled spell learning type encountered, falling back to memo table:", table)
	SpellLearnTable = GemRB.LoadTable (SpellLearnTable)

	CastingStatValue = 0
	if IWD2:
		# mxsplbon.2da is handled in core, but does also affect learning, at least in chargen
		BonusSpellTable = GemRB.LoadTable ("mxsplbon")
		ClassRowName = GUICommon.GetClassRowName (pc)
		CastingStat = CommonTables.ClassSkills.GetValue (ClassRowName, "CASTING", GTV_INT)
		CastingStatValue = GemRB.GetPlayerStat (pc, CastingStat)

	AlreadyShown = 0
	for i in range (9):
		# make sure we always have a value to minus (bards)
		SecondPoints = SpellsToMemoTable.GetValue (str(level-diff), str(i+1), GTV_INT)

		# make sure we get more spells of each class before continuing
		SpellsSelectPointsLeft[i] = SpellsToMemoTable.GetValue (str(level), str(i+1), GTV_INT) - SecondPoints
		if SpellsSelectPointsLeft[i] <= 0:
			continue

		SpellsSelectPointsLeft[i] = SpellLearnTable.GetValue (str(level), str(i+1), GTV_INT)
		# luckily the bonus applies both to learning and memorization
		if IWD2 and chargen:
			BonusPoints[i] = BonusSpellTable.GetValue (str(CastingStatValue), str(i+1), GTV_INT)
			SpellsSelectPointsLeft[i] += BonusPoints[i]

		if SpellsSelectPointsLeft[i] <= 0:
			continue
		elif chargen and KitMask != 0x4000 and (not IWD2 or SpellBookType == IE_IWD2_SPELL_WIZARD):
			# specialists get an extra spell per level
			SpellsSelectPointsLeft[i] += 1
			BonusPoints[i] += 1

		# get all the spells of the given level
		Spells[i] = Spellbook.GetMageSpells (KitMask, GemRB.GetPlayerStat (pc, IE_ALIGNMENT), i+1, Class)

		# dump all the spells we already know
		NumDeleted = 0
		for j in range (len (Spells[i])):
			CurrentIndex = j - NumDeleted # this ensure we don't go out of range
			if Spellbook.HasSpell (pc, SpellBookType, i, Spells[i][CurrentIndex][0]) >= 0:
				del Spells[i][CurrentIndex]
				NumDeleted += 1

		# display these spells if it's the first non-zero level
		if AlreadyShown == 0:
			# save the level and spellbook data
			SpellLevel = i
			SpellBook = [0]*len(Spells[i])

			ScrollBar = SpellsWindow.GetControl (NewScrollBarID)
			UpdateScrollBar (ScrollBar, len (Spells[i]))

			# show our spells
			ShowSpells ()
			AlreadyShown = 1

	# show the selection window
	if chargen:
		if recommend:
			SpellsWindow.Focus()
		else:
			SpellsWindow.ShowModal (MODAL_SHADOW_GRAY)
	else:
		SpellsWindow.ShowModal (MODAL_SHADOW_GRAY)

	return

def UpdateScrollBar (ScrollBar, SpellCount):
	if not ScrollBar:
		return

	# only scroll if we have more spells than buttons
	# that's 25 if the extra 25th spell slot is available in sorcerer level up
	if SpellCount > ButtonCount + ExtraSpellButtons():
		ScrollBar.SetVisible (True)
		ScrollBar.OnChange (ShowSpells)

		extraCount = SpellCount - ButtonCount
		if chargen:
			count = GUICommon.ceildiv (extraCount, 6) + 1
		else: # there are five rows of 5 spells in level up of sorcerers
			count = GUICommon.ceildiv (extraCount - 1, 5) + 1
		ScrollBar.SetVarAssoc ("SpellTopIndex", count, 0, count)
	else:
		ScrollBar.SetVarAssoc ("SpellTopIndex", 0)
		ScrollBar.SetVisible (False)
		ScrollBar.OnChange (None)

def SpellsDonePress ():
	"""Move to the next assignable level.

	If there is not another assignable level, then save all the new spells and
	close the window."""

	global SpellBook, SpellLevel, SpellsWindow, MemoBook, Memorization

	# oops, we were here before, just memorise the spells and exit
	if sum(MemoBook) > 0:
		for i in MemoBook:
			if i:
				GemRB.MemorizeSpell(pc, SpellBookType, SpellLevel, i-1, 1)
		SpellBook = []
		MemoBook = [0]*ButtonCount

	# save all the spells
	if not Memorization:
		sbt = SpellBookType if IWD2 else -1
		for i in range (len (Spells[SpellLevel])):
			if SpellBook[i]: # we need to learn this spell
				GemRB.LearnSpell (pc, Spells[SpellLevel][i][0], 0, sbt)

		# check to see if we need to update again
		for i in range (SpellLevel+1, 9):
			if SpellsSelectPointsLeft[i] > 0:
				# reset the variables
				GemRB.SetVar ("SpellTopIndex", 0)
				SpellLevel = i
				if not (chargen and GameCheck.IsBG1()):
					SpellBook = [0]*len(Spells[i])

				ScrollBar = SpellsWindow.GetControl (NewScrollBarID)
				UpdateScrollBar (ScrollBar, len (Spells[i]))

				# show the spells and set the done button to off
				ShowSpells ()
				DoneButton.SetDisabled(True)
				return

		# bg1 lets you memorize spells too (iwd too, but it does it by itself)
		# tob doesn't, but we could enhance it — need to add multilevel support
		# enable below if GameCheck.IsTOB () and not Spellbook.HasSorcererBook (pc)
		if chargen and sum(MemoBook) == 0 and \
		(GameCheck.IsBG1() or (IWD2 and SpellBookType == IE_IWD2_SPELL_WIZARD)):
			SpellLevel = 0
			if GameCheck.IsTOB ():
				SpellsSelectPointsLeft[SpellLevel] = GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_WIZARD, SpellLevel, 1)
			else:
				# bump it for specialists and iwd2 casters with high stats
				SpellsSelectPointsLeft[SpellLevel] = 1 + BonusPoints[SpellLevel]
			# FIXME: setting the proper count here breaks original characters, see #680
			#GemRB.SetMemorizableSpellsCount (pc, SpellsSelectPointsLeft[SpellLevel], SpellBookType, SpellLevel)
			DoneButton.SetDisabled (True)
			Memorization = 1
			ShowKnownSpells()
			return

	# close our window and update our records
	if SpellsWindow and (not chargen or GameCheck.IsBG2OrEE () or IWD2):
		SpellsWindow.Close ()
		SpellsWindow = None

	# move to the next script if this is chargen
	if chargen:
		if GameCheck.IsBG2OrEE ():
			GemRB.SetNextScript("GUICG6")
		elif GameCheck.IsBG1():
			SpellsWindow.Close ()
			# HACK
			from CharGenCommon import next
			next()
		elif IWD2:
			GemRB.SetNextScript("CharGen7")
	elif IWD2:
		import GUIREC
		GUIREC.FinishLevelUp ()

	return

def ShowKnownSpells ():
	"""Shows the viewable 24 spells."""

	j = RowIndex()
	Spells[SpellLevel] = Spellbook.GetMageSpells (KitMask, GemRB.GetPlayerStat (pc, IE_ALIGNMENT), SpellLevel+1, Class)

	# reset the title
	#17224 for priest spells
	Title = SpellsWindow.GetControl (0x10000000)
	Title.SetText(17189)

	# we have a grid of 24 (usually) spells
	for i in range (ButtonCount):
		# ensure we can learn this many spells
		SpellButton = SpellsWindow.GetControl (i+SpellStart)
		if i + j >= len (SpellBook) or not SpellBook[i+j]:
			SpellButton.SetState (IE_GUI_BUTTON_DISABLED)
			SpellButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
			SpellButton.OnPress (None)
			continue
		else:
			SpellButton.SetState (IE_GUI_BUTTON_ENABLED)
			SpellButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)

		# fill in the button with the spell data
		Spell = GemRB.GetSpell (Spells[SpellLevel][i+j][0], 1)
		SpellButton.SetTooltip(Spell['SpellName'])
		SpellButton.SetValue (i)
		SpellButton.OnPress (MemorizePress)
		if GameCheck.IsBG2OrEE ():
			SpellButton.SetSprites("GUIBTBUT",0, 0,1,2,3)
		else:
			SpellButton.SetSprites("GUIBTBUT",0, 0,1,24,25)
		SpellButton.SetBorder (0, None, 0,0)

		SpellButton.SetSpellIcon(Spells[SpellLevel][i+j][0], 1)
		SpellButton.SetFlags(IE_GUI_BUTTON_PICTURE, OP_OR)

	# show which spells are selected
	ShowSelectedSpells ()

	GemRB.SetToken("number", str(SpellsSelectPointsLeft[SpellLevel]))
	SpellsTextArea.SetText(17253)

	if SpellsPickButton == None:
		# no recommendations at all
		return

	if Memorization == 1:
		SpellsPickButton.SetState (IE_GUI_BUTTON_DISABLED)
	else:
		SpellsPickButton.SetState (IE_GUI_BUTTON_ENABLED)

	return

def MemorizePress (btn):
	"""Toggles the memorisation of the given spell."""

	global SpellsSelectPointsLeft, Spells, SpellBook, MemoBook

	# get our variables
	j = RowIndex()
	i = btn.Value + j

	# get the spell that's been pushed
	Spell = GemRB.GetSpell (Spells[SpellLevel][i][0], 1)
	SpellsTextArea.SetText (Spell["SpellDesc"])

	# make sure we can learn the spell
	if MemoBook[i]: # already picked -- unselecting
		SpellsSelectPointsLeft[SpellLevel] = SpellsSelectPointsLeft[SpellLevel] + 1
		MemoBook[i] = 0
		DoneButton.SetDisabled (True)
	else: # selecting
		# we don't have any picks left
		if SpellsSelectPointsLeft[SpellLevel] == 0:
			MarkButton (i, 0)
			return

		# select the spell and change the done state if need be
		SpellsSelectPointsLeft[SpellLevel] = SpellsSelectPointsLeft[SpellLevel] - 1
		MemoBook[i] = Spellbook.HasSpell(pc, SpellBookType, SpellLevel, Spells[SpellLevel][i][0]) + 1 # so all values are above 0
		if SpellsSelectPointsLeft[SpellLevel] == 0:
			DoneButton.SetDisabled (False)

	# show selected spells
	ShowSelectedSpells ()

	return

def ShowSpells ():
	"""Shows the viewable 24 spells."""

	j = RowIndex()

	# we have a grid of 24 spells
	extraButtons = ExtraSpellButtons ()
	for i in range (ButtonCount + extraButtons):
		# ensure we can learn this many spells
		SpellButton = SpellsWindow.GetControl (i+SpellStart)
		if i + j >= len (Spells[SpellLevel]):
			SpellButton.SetState (IE_GUI_BUTTON_DISABLED)
			SpellButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
			continue
		else:
			SpellButton.SetState (IE_GUI_BUTTON_ENABLED)
			SpellButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)

		# fill in the button with the spell data
		Spell = GemRB.GetSpell (Spells[SpellLevel][i+j][0], 1)
		SpellButton.SetTooltip(Spell['SpellName'])
		SpellButton.SetVarAssoc("ButtonPressed", i)
		SpellButton.OnPress (SpellsSelectPress)
		if GameCheck.IsBG2OrEE ():
			SpellButton.SetSprites("GUIBTBUT",0, 0,1,2,3)
		else:
			SpellButton.SetSprites("GUIBTBUT",0, 0,1,24,25)

		SpellButton.SetSpellIcon(Spells[SpellLevel][i+j][0], 1)
		SpellButton.SetFlags(IE_GUI_BUTTON_PICTURE, OP_OR)

		# don't allow the selection of an un-learnable spell
		if Spells[SpellLevel][i+j][1] == 0:
			SpellButton.SetState(IE_GUI_BUTTON_LOCKED)
			# shade red
			color = {'r' : 200, 'g' : 0, 'b' : 0, 'a' : 100}
			SpellButton.SetBorder (0, color, 1, 1)
		elif Spells[SpellLevel][i+j][1] == 1: # learnable
			SpellButton.SetState (IE_GUI_BUTTON_ENABLED)
			# unset any borders on this button or an un-learnable from last level
			# will still shade red even though it is clickable
			SpellButton.SetBorder (0, None, 0,0)
		else: # specialist (for iwd2 which has no green frames)
			# use the green border state for matching specialist spells
			color = {'r' : 0, 'g' : 200, 'b' : 0, 'a' : 100}
			SpellButton.SetBorder (0, color, 1,0)
			SpellButton.SetState (IE_GUI_BUTTON_FAKEDISABLED)

	# show which spells are selected
	ShowSelectedSpells ()

	GemRB.SetToken("number", str(SpellsSelectPointsLeft[SpellLevel]))
	SpellsTextArea.SetText(17250)
	LevelLabel = SpellsWindow.GetControl (0x10000000)
	if LevelLabel:
		GemRB.SetToken ("SPELLLEVEL", str(SpellLevel+1))
		LevelLabel.SetText (10345)

	return

def SpellsSelectPress (btn):
	"""Toggles the selection of the given spell."""

	global SpellsSelectPointsLeft, Spells, SpellBook

	# get our variables
	j = RowIndex()
	i = btn.Value + j

	# get the spell that's been pushed
	Spell = GemRB.GetSpell (Spells[SpellLevel][i][0], 1)
	SpellsTextArea.SetText (Spell["SpellDesc"])

	# make sure we can learn the spell
	if Spells[SpellLevel][i][1]:
		if SpellBook[i]: # already picked -- unselecting
			SpellsSelectPointsLeft[SpellLevel] = SpellsSelectPointsLeft[SpellLevel] + 1
			SpellBook[i] = 0
			DoneButton.SetDisabled (True)
		else: # selecting
			# we don't have any picks left
			if SpellsSelectPointsLeft[SpellLevel] == 0:
				MarkButton (i, 0)
				return

			# if we have a specialist, we must make sure they pick at least
			# one spell of their school per level
			if SpellsSelectPointsLeft[SpellLevel] == 1 and not HasSpecialistSpell () \
			and Spells[SpellLevel][i][1] != 2:
				SpellsTextArea.SetText (33381)
				MarkButton (i, 0)
				return

			# select the spell and change the done state if need be
			SpellsSelectPointsLeft[SpellLevel] = SpellsSelectPointsLeft[SpellLevel] - 1
			SpellBook[i] = 1
			if SpellsSelectPointsLeft[SpellLevel] == 0:
				DoneButton.SetDisabled (False)

	# show selected spells
	ShowSelectedSpells ()

	return

def MarkButton (i, select):
	"""Shows enabled, disabled, or highlighted button.

	If selected is true, the button is highlighted.
	Be sure i is sent with +SpellTopIndex!"""

	j = RowIndex()

	if select:
		state = IE_GUI_BUTTON_SELECTED
	else:
		if Spells[SpellLevel][i][1] == 1:
			state = IE_GUI_BUTTON_ENABLED
		elif Spells[SpellLevel][i][1] == 2:
			# specialist spell
			state = IE_GUI_BUTTON_FAKEDISABLED
		else: # can't learn
			state = IE_GUI_BUTTON_LOCKED

	# we have to use the index on the actual grid
	SpellButton = SpellsWindow.GetControl(i+SpellStart-j)
	SpellButton.SetState(state)
	return

def ShowSelectedSpells ():
	"""Highlights all selected spells."""

	k = RowIndex()

	# mark all of the spells picked thus far
	for j in range (ButtonCount + ExtraSpellButtons()):
		if j + k >= len (SpellBook): # make sure we don't call unavailable indexes
			break
		if (SpellBook[j+k] and not Memorization) or (Memorization and MemoBook[j+k]): # selected
			MarkButton (j+k, 1)
		else: # not selected
			MarkButton (j+k, 0)

	# show how many spell picks are left
	SpellPointsLeftLabel.SetText (str (SpellsSelectPointsLeft[SpellLevel]))
	return

def SpellsCancelPress ():
	"""Removes all known spells and close the window.

	This is only callable within character generation."""

	# remove all learned spells
	Spellbook.RemoveKnownSpells (pc, SpellBookType, 1, 9, 1)

	if GameCheck.IsBG2OrEE ():
		# unload teh window and go back
		if SpellsWindow:
			SpellsWindow.Close ()
		GemRB.SetNextScript("CharGen6") #haterace
	elif GameCheck.IsBG1():
		if SpellsWindow:
			SpellsWindow.Close ()
		import CharGenCommon
		CharGenCommon.back()
	elif IWD2:
		if SpellsWindow:
			SpellsWindow.Close ()
		GemRB.SetNextScript("Feats")
	else:
		print("Uh-oh in SpellsCancelPress in", GemRB.GameType)
	return

def SpellsPickPress ():
	"""Auto-picks spells for the current level based on splautop.2da.

	Only used in character generation.
	Perhaps implement for sorcerers, if possible."""

	global SpellBook, SpellsSelectPointsLeft

	# load up our table
	if Spellbook.HasSorcererBook (pc) & 3 == 3:
		# priestly sorcerer like shaman
		AutoTable = GemRB.LoadTable ("splautos")
	else:
		AutoTable = GemRB.LoadTable ("splautop")

	for i in range (AutoTable.GetRowCount ()):
		if SpellsSelectPointsLeft[SpellLevel] == 0:
			break

		CurrentSpell = AutoTable.GetValue (i, SpellLevel, GTV_STR)
		for j in range (len (Spells[SpellLevel])):
			# we can learn the spell, and it's not in our book
			if Spells[SpellLevel][j][0].upper() == CurrentSpell.upper() \
			and Spells[SpellLevel][j][1] and not SpellBook[j]:
				# make sure we learn at least 1 specialist spell
				if SpellsSelectPointsLeft[SpellLevel] == 1 and not HasSpecialistSpell () \
				and Spells[SpellLevel][j][1] != 2:
					SpellsTextArea.SetText (33381)
					break

				# save our spell and decrement the points left
				SpellBook[j] = 1
				SpellsSelectPointsLeft[SpellLevel] -= 1
				break

	# show the spells and update the counts
	ShowSelectedSpells ()

	# if we don't have any points left, we can enable the done button
	if not SpellsSelectPointsLeft[SpellLevel]:
		DoneButton.SetDisabled (False)

	return

def HasSpecialistSpell ():
	"""Determines if specialist requirements have been met.

	Always returns true if the mage is not a specialist.
	Returns true only if the specialists knows at least one spell from their
	school."""

	# always return true for non-kitted classed
	if KitMask == 0x4000:
		return 1

	# return true if we've memorized a school spell of this level
	for i in range (len (Spells[SpellLevel])):
		if Spells[SpellLevel][i][1] == 2 and SpellBook[i]:
			return 1

	# return true if there are no specialist spells of this level
	SpecialistSpellCount = 0
	for i in range (len (Spells[SpellLevel])):
		if Spells[SpellLevel][i][1] == 2:
			SpecialistSpellCount = 1
			break
	if SpecialistSpellCount == 0:
		return 1

	# no luck
	return 0

def RowIndex ():
	"""Determines which factor to use in scrolling of spells

	It depends on if it is character generation where you have
	4 rows of 6 spells (24), or it is sorcs level up window where there
	is 4 rows of 5 spells and 5th row of 4 spell, but you may also use 25th slot there
	and it is 5 rows of 5 with 25 spells seen at once. """

	SpellTopIndex = GemRB.GetVar ("SpellTopIndex")
	if chargen:
		return ( SpellTopIndex + 1 ) * 6 - 6
	elif IWD2: # 30 during level-up
		return ( SpellTopIndex + 1 ) * 6 - 6
	elif EnhanceGUI:
		return ( SpellTopIndex + 1 ) * 5 - 5
	else:
		return SpellTopIndex

def ExtraSpellButtons ():
	"""Determines if extra spell slots are available. """

	if EnhanceGUI and (not chargen):
		return 1
	else:
		return 0



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# Single Player Party Formation (GUISP)

import GemRB
from GUIDefines import *

PartyFormationWindow = 0
ExitWindow = 0
ReviewWindow = 0

def OnLoad ():
	global PartyFormationWindow

	PartyFormationWindow = GemRB.LoadWindow (0, "GUISP")
	ExitButton = PartyFormationWindow.GetControl (30)
	ExitButton.SetText (13906)
	ExitButton.OnPress (ExitPress)
	ExitButton.MakeEscape()

	ModifyCharactersButton = PartyFormationWindow.GetControl (43)
	ModifyCharactersButton.SetText (18816)
	ModifyCharactersButton.SetState (IE_GUI_BUTTON_DISABLED)
	ModifyCharactersButton.OnPress (None) #TODO: ModifyCharactersPress

	DoneButton = PartyFormationWindow.GetControl (28)
	DoneButton.SetText (11973)
	DoneButton.MakeDefault()

	Portraits = 0

	for i in range (18,24):
		Label = PartyFormationWindow.GetControl (0x10000012+i)
		#removing this label, it just disturbs us
		Label.SetSize (0, 0)
		Button = PartyFormationWindow.GetControl (i-12)
		portrait = GemRB.GetPlayerPortrait (i-17, 1)
		if portrait is None:
			Button.SetFlags (IE_GUI_BUTTON_NORMAL,OP_SET)
			Button.OnPress (GeneratePress)
		else:
			Button.SetPicture (portrait["ResRef"])
			Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_OR)
			Portraits = Portraits+1
			Button.OnPress (ReviewPress)

		Button.SetVarAssoc ("Slot",i-17)

		Button = PartyFormationWindow.GetControl (i)
		Button.SetVarAssoc ("Slot",i-17)
		if portrait is None:
			Button.SetText (10264)
			Button.OnPress (GeneratePress)
		else:
			Button.SetText (GemRB.GetPlayerName (i-17,0) )
			Button.OnPress (ReviewPress)

	if Portraits == 0:
		DoneButton.SetState (IE_GUI_BUTTON_DISABLED)
	else:
		DoneButton.SetState (IE_GUI_BUTTON_ENABLED)
		DoneButton.MakeDefault()
	DoneButton.OnPress (EnterGamePress)

	PartyFormationWindow.ShowModal (MODAL_SHADOW_NONE)
	GemRB.GetView ("STARTWIN").SetDisabled (False)
	return

def ExitPress ():
	global PartyFormationWindow, ExitWindow
	ExitWindow = GemRB.LoadWindow (7, "GUISP")

	TextArea = ExitWindow.GetControl (0)
	TextArea.SetText (11329)

	CancelButton = ExitWindow.GetControl (2)
	CancelButton.SetText (13727)
	CancelButton.MakeEscape()
	CancelButton.OnPress (ExitCancelPress)

	DoneButton = ExitWindow.GetControl (1)
	DoneButton.SetText (11973)
	DoneButton.MakeDefault()
	DoneButton.OnPress (ExitDonePress)

	ExitWindow.ShowModal (MODAL_SHADOW_GRAY)
	return

def ExitDonePress ():
	global PartyFormationWindow, ExitWindow
	if ExitWindow:
		ExitWindow.Close ()
	if PartyFormationWindow:
		PartyFormationWindow.Close ()
	GemRB.SetNextScript ("Start")
	return

def ExitCancelPress ():
	global PartyFormationWindow, ExitWindow
	if ExitWindow:
		ExitWindow.Close ()
	PartyFormationWindow.Focus()
	return

def GeneratePress ():
	global PartyFormationWindow
	if PartyFormationWindow:
		PartyFormationWindow.Close ()
	GemRB.SetNextScript ("CharGen")
	return

def EnterGamePress ():
	GemRB.SetToken ("SaveDir", "mpsave") # iwd2 is always using 'mpsave'
	GemRB.EnterGame ()
	return

def ReviewPress ():
	global PartyFormationWindow, ReviewWindow

	ReviewWindow = GemRB.LoadWindow (8, "GUISP")

	DoneButton = ReviewWindow.GetControl (1)
	DoneButton.SetText (11973)
	DoneButton.MakeDefault()
	DoneButton.MakeEscape()
	DoneButton.OnPress (ReviewDonePress)

	LeftTextArea = ReviewWindow.GetControl (2)
	RightTextArea = ReviewWindow.GetControl (3)

	MyChar = GemRB.GetVar ("Slot")
	import GUIREC
	GUIREC.DisplayGeneral (MyChar, LeftTextArea)
	GUIREC.DisplaySkills (MyChar, RightTextArea)

	ReviewWindow.ShowModal (MODAL_SHADOW_GRAY)
	return

def ReviewDonePress ():
	global PartyFormationWindow, ReviewWindow

	if ReviewWindow:
		ReviewWindow.Close ()
	PartyFormationWindow.Focus()
	return




# GemRB - Infinity Engine Emulator
# Copyright (C) 2010 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# a library of any functions that are common to bg1 and bg2, but not useful for others

import GemRB
import CommonTables
import GUICommon
from ie_stats import IE_ALIGNMENT, IE_RACE, IE_REPUTATION, IE_SEX

def RefreshPDoll(button, MinorColor, MajorColor, SkinColor, HairColor):
	MyChar = GemRB.GetVar ("Slot")
	AnimID = 0x6000
	table = GemRB.LoadTable ("avprefr")
	Race = GemRB.GetPlayerStat (MyChar, IE_RACE)
	AnimID = AnimID + table.GetValue(Race, 0)
	table = GemRB.LoadTable ("avprefc")
	ClassName = GUICommon.GetClassRowName (MyChar)
	AnimID = AnimID + table.GetValue (ClassName, "CLASS")
	table = GemRB.LoadTable ("avprefg")
	Gender = GemRB.GetPlayerStat (MyChar, IE_SEX)
	AnimID = AnimID + table.GetValue (Gender, 0)
	ResRef = CommonTables.Pdolls.GetValue (hex(AnimID), "LEVEL1")

	button.SetPLT (ResRef, 0, MinorColor, MajorColor, SkinColor, 0, 0, HairColor, 0)
	return

def SetReputation ():
	MyChar = GemRB.GetVar ("Slot")

	# If Rep > 0 then MyChar is an imported character with a saved reputation,
	# in which case the reputation shouldn't be overwritten.
	Rep = GemRB.GetPlayerStat (MyChar, IE_REPUTATION)

	if Rep <= 0:
		# use the alignment to apply starting reputation
		Alignment = GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT)
		AlignmentAbbrev = CommonTables.Aligns.FindValue (3, Alignment)
		RepTable = GemRB.LoadTable ("repstart")
		Rep = RepTable.GetValue (AlignmentAbbrev, 0) * 10
		GemRB.SetPlayerStat (MyChar, IE_REPUTATION, Rep)

	# set the party rep if this is the main char
	if MyChar == 1:
		GemRB.GameSetReputation (Rep)



# GemRB - Infinity Engine Emulator
# Copyright (C) 2007 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# QuitGame.py - display EndGame sequence

###################################################

import GemRB
from GUIDefines import *
import GUICommon
import CommonWindow

def OnLoad ():
	movies = ["", "T1DEATH", "T1ABSORB", "FINALE"]

	CommonWindow.SetGameGUIHidden(True)
	which = GemRB.GetVar ("QuitGame1")
	if which is not None:
		GemRB.PlayMovie (movies[which], 1)
	which = GemRB.GetVar ("QuitGame2")
	if which is not None:
		GemRB.PlayMovie (movies[which], 1)
	which = GemRB.GetVar ("QuitGame3")
	if which:
		DeathWindowEnd ()
	else:
		GemRB.QuitGame ()
		GemRB.SetNextScript("Start")

def DonePress ():
		GemRB.QuitGame ()
		GemRB.SetNextScript("Start")

def DeathWindowEnd ():
	GemRB.GamePause (1,3)

	Window = GemRB.LoadWindow (25, GUICommon.GetWindowPack())

	#reason for death
	Label = Window.GetControl (0x0fffffff)
	strref = GemRB.GetVar ("QuitGame3")
	Label.SetText (strref)

	#done
	Button = Window.GetControl (1)
	Button.SetText (17237)
	Button.OnPress (DonePress)
	Button.MakeDefault()

	#making the playing field gray
	GUICommon.GameWindow.SetDisabled(True)
	return




from __future__ import print_function # so that print(blah) will work in console

from ie_stats import *
from GUIDefines import *
import GemRB
from GemRB import * # we dont want to have to type 'GemRB.Command' instead of simply 'Command'

def OnLoad():
	consoleWin = GemRB.LoadWindow(0, "console", WINDOW_TOP|WINDOW_HCENTER)
	consoleWin.SetFlags(IE_GUI_VIEW_INVISIBLE | WF_BORDERLESS | WF_ALPHA_CHANNEL, OP_OR)
	consoleWin.SetFlags(WF_DESTROY_ON_CLOSE, OP_NAND);
	consoleWin.AddAlias("WIN_CON")
	consoleWin.SetBackground({'r' : 0, 'g' : 0, 'b' : 0, 'a' : 200})
	
	histLabel = consoleWin.GetControl(2)
	histLabel.SetText ("History:")
	hist = consoleWin.GetControl(3)

	console = consoleWin.ReplaceSubview (0, IE_GUI_CONSOLE, hist)
	console.AddAlias ("CONSOLE_CTL", 1);
	
	consoleWin.SetAction(console.Focus, ACTION_WINDOW_FOCUS_GAINED)
	
	consoleOut = consoleWin.GetControl(1)
	consoleOut.SetFlags (IE_GUI_TEXTAREA_AUTOSCROLL)
	consoleOut.AddAlias("CONSOLE", 1);
	consoleOut.SetBackground({'r' : 0, 'g' : 0, 'b' : 0, 'a' : 200})
	
def ToggleConsole():
	consoleWin = GemRB.GetView("WIN_CON")
	if consoleWin is None: # if outside of a game
		return

	if consoleWin.IsVisible():
		consoleWin.Close()
	else:
		consoleWin.Focus()

# /handy/ shorthand forms
def gps(stat, base=0):
	print (GemRB.GetPlayerStat(GemRB.GameGetFirstSelectedPC(), stat, base))

def sps(stat, value, pcf=1):
	GemRB.SetPlayerStat(GemRB.GameGetFirstSelectedPC(), stat, value, pcf)

def mta(area):
	GemRB.MoveToArea(area)

def debug(level):
	GemRB.ConsoleWindowLog (level)
	
def cast(spellRes):
	GemRB.SpellCast (GemRB.GameGetFirstSelectedPC (), -3, 0, spellRes)

def cc(cre, px=-1, py=-1):
	GemRB.CreateCreature(GemRB.GameGetFirstSelectedPC(), cre, px, py)

def ci(item, c0=1, c1=0, c2=0, slot=-1):
	GemRB.CreateItem(GemRB.GameGetFirstSelectedPC(), item, slot, c0, c1, c2)

def cv(var, context="GLOBAL"):
	GemRB.CheckVar(var, context)

def ex(cmd, runner = 0):
	GemRB.ExecuteString(cmd, runner)

def ev(trigger):
	GemRB.EvaluateString(trigger)

# the actual function that the GemRB::Console calls
def Exec(cmd):
	import sys
	
	con = GemRB.GetView("CONSOLE", 1)
	
	class OutputCapture(object):
		def __init__(self, out):
			self.out = out
			self.buffer = ""
		
		def write(self, message):
			self.out.write(message)
			self.buffer += str(message)
			if self.buffer.endswith("\n"):
				out = self.buffer
				if out:
					con.Append("[color=ffffff]" + cmd + ": [/color][color=00ff00]" + out + "[/color]\n")
				self.buffer = ""
	
	try:
		stdout = sys.stdout

		if con:
			sys.stdout = OutputCapture(stdout)

		locals = {} # we dont want to expose our locals
		modend = cmd.find('.')
		if modend > -1 and modend < len("ex('ActionOverride(Pla"): # Point arguments use a dot
			import importlib
			importlib.invalidate_caches()
			modname = cmd[0:modend]
			locals[modname] = importlib.import_module(modname)
		return eval(cmd, globals(), locals)
	except (SyntaxError, NameError, TypeError, ZeroDivisionError) as error:
		if con:
			con.Append("[color=ffffff]" + cmd + ": [/color][color=ff0000]" + str(error) + "[/color]\n")
			
		sys.stderr.write(error)
	finally:
		sys.stdout = stdout



# !!! Keep these synchronized with Spellbook.h !!!

# LearnSpell flags
LS_ADDXP = 1   #give xp for learning it
LS_LEARN = 2   #give message when learned it
LS_STATS = 4   #check stats (alignment, etc)
LS_MEMO  = 8   #memorize it instantly (add innate)
LS_NOXP  = 16  #don't give xp; has precedence over LS_ADDXP

# LearnSpell return values
LSR_OK      = 0
LSR_KNOWN   = 1  #already knows
LSR_INVALID = 2  #invalid resref
LSR_FAILED  = 3  #failed stat roll
LSR_STAT    = 4  #insufficient stat (can't learn the spell due to low stat)
LSR_LEVEL   = 5  #insufficient level (low mage, etc level)
LSR_FULL    = 6  #can't learn more spells of this level (due to level)

# CheckSpecialSpell return values (see Interface.h)
SP_IDENTIFY = 1  # any spell that cannot be cast from the menu
SP_SILENCE  = 2  # any spell that can be cast in silence
SP_SURGE    = 4  # any spell that cannot be cast during a wild surge
SP_REST     = 8  # any spell that is cast upon rest if memorized
